c***** fcdf.for ***** version: 23 Jun 2008 *****************************
c F-cumulative distribution function, derived from stat
c   relations in Abramowitz and Stegun's "Handbook of Mathematical
c   Functions" via James Filliben's coding.
c***********************************************************************
      real*8 function fcdf(x,nu1,nu2)
      implicit none
c
c Arguments:
c
      real*8 nu1,nu2,x
c
c Function Declarations:
c
      real*8 chscdf,norcdf
c
c Local Variables:
c
      real*8 a,ai,amean,arg,b,coef,coef1,coef2,costh,dden,dfact1,dfact2
      real*8 dnum,dpow1,dpow2,nucut1,nucut2,pi,sd,sinth,sum
      real*8 t1,t2,t3,term,term1,term2,term3,theta,u,z,zratio
      integer i,ievodd,iflag1,iflag2,imax,imin
c
c Data:
c
      data pi/3.14159265358979d0/
      data dpow1,dpow2/0.33333333333333d0,0.66666666666667d0/
      data nucut1,nucut2/1d2,1d3/
c
c Check input arguments for errors
c
      if(nu1.le.0d0)then
        pause 'fcdf error: Non-positive M'
        fcdf=0d0
        return
      endif
c
      if(nu2.le.0d0)then
        pause 'fcdf error: Non-positive N'
        fcdf=0d0
        return
      endif
c
      if(x.lt.0d0)then
        pause 'fcdf error: Negative X'
        fcdf=0d0
        return
      endif
c
      if(nu2.gt.4d0)then
        t1=2d0/nu1
        t2=nu2/(nu2-2d0)
        t3=(nu1+nu2-2d0)/(nu2-4d0)
        amean=t2
        sd=dsqrt(t1*t2*t2*t3)
        zratio=(x-amean)/sd
        if(nu2.lt.1d1.and.zratio.lt.-3d3)then
          fcdf=0d0
          return
        endif
        if(nu2.ge.1d1.and.zratio.lt.-1.5d2)then
          fcdf=0d0
          return
        endif
        if(nu2.lt.1d1.and.zratio.gt.3d3)then
          fcdf=1.0
          return
        endif
        if(nu2.ge.1d1.and.zratio.gt.1.5d2)then
          fcdf=1.0
          return
        endif
      endif
c
      if(nu1.lt.nucut2.and.nu2.lt.nucut2)then
        z=nu2/(nu2+nu1*x)
        iflag1=idnint(nu1)-2*(idnint(nu1)/2)
        iflag2=idnint(nu2)-2*(idnint(nu2)/2)
        if(iflag1.eq.0)then
          sum=0d0
          term=1d0
          imax=(idnint(nu1)-2)/2
          if(imax.le.0)then
            sum=sum+1d0 
            sum=(z**(nu2/2d0))*sum
            fcdf=1d0-sum 
            return
          endif
          do i=1,imax 
            ai=dfloat(i)
            coef1=2d0*(ai-1d0)
            coef2=2d0*ai
            term=term*((nu2+coef1)/coef2)*(1d0-z)
            sum=sum+term
          enddo
        endif
        if(iflag2.eq.0)then
          sum=0d0
          term=1d0
          imax=(idnint(nu2)-2)/2
          if(imax.le.0)then
            sum=sum+1d0 
            fcdf=((1d0-z)**(nu1/2d0))*sum 
            return
          endif
          do i=1,imax 
            ai=dfloat(i)
            coef1=2d0*(ai-1d0)
            coef2=2d0*ai
            term=term*((nu1+coef1)/coef2)*z
            sum=sum+term
          enddo
        endif
        sum=0d0
        term=1d0
        arg=dsqrt((nu1/nu2)*x)
        theta=datan(arg)
        sinth=arg/dsqrt(1d0+arg*arg)
        costh=1d0/dsqrt(1d0+arg*arg)
        if(nu2.ne.1d0)then
          if(nu2.ne.3d0)then
            imax=idnint(nu2)-2
            do i=3,imax,2
              ai=dfloat(i)
              coef1=ai-1d0
              coef2=ai
              term=term*(coef1/coef2)*(costh*costh)
              sum=sum+term
            enddo
          endif
          sum=sum+1d0 
          sum=sum*sinth*costh
        endif
        a=(2d0/pi)*(theta+sum)
        sum=0d0
        term=1d0
        if(nu1.eq.1d0)b=0d0
        if(nu1.ne.1d0)then
          if(nu1.eq.3d0)then
            imax=idnint(nu1)-3
            do i=1,imax,2
              ai=dfloat(i)
              coef1=ai
              coef2=ai+2d0
              term=term*((nu2+coef1)/coef2)*(sinth*sinth)
              sum=sum+term
            enddo
          endif
c
          sum=sum+1d0 
          sum=sum*sinth*(costh**idnint(nu2))
          coef=1d0
          ievodd=idnint(nu2)-2*(idnint(nu2)/2)
          imin=3
          if(ievodd.eq.0)imin=2
          if(imin.le.idnint(nu2))then
            do i=imin,idnint(nu2),2
              ai=dfloat(i)
              coef=((ai-1d0)/ai)*coef
            enddo
          endif
c
          coef=coef*nu2
          if(ievodd.ne.0)coef=coef*(2d0/pi)
          b=coef*sum
        endif
        fcdf=a-b
        return
c
      elseif(nu1.lt.nucut1.and.nu2.ge.nucut2)then
        term1=nu1
        term2=(nu1/nu2)*(0.5d0*nu1-1d0)
        term3=-(nu1/nu2)*0.5d0
        u=(term1+term2)/((1d0/x)-term3)
        fcdf=chscdf(u,nu1) 
        return
      elseif(nu1.ge.nucut2.and.nu2.lt.nucut1)then
        term1=nu2
        term2=(nu2/nu1)*(0.5d0*nu2-1d0)
        term3=-(nu2/nu1)*0.5d0
        u=(term1+term2)/(x-term3)
        fcdf=1d0-chscdf(u,nu2)
        return
      endif
c
c All other feasible cases:
      dfact1=1d0/(4.5d0*nu1)
      dfact2=1d0/(4.5d0*nu2)
      dnum=((1d0-dfact2)*(x**dpow1))-(1d0-dfact1)
      dden=dsqrt((dfact2*(x**dpow2))+dfact1)
      u=dnum/dden
      fcdf=norcdf(u)
      return
c
      end 

c***** norcdf.for ***** version: 22 Jun 2008 ***************************
c Cumulative distribution function for the normal (gaussian) 
c   distribution with mean = 0 and standard deviation = 1, derived from 
c   stat relations in the NBS applied math series and Johnson & Kotz,
c   via James Filliben's coding.
c***********************************************************************
      real*8 function norcdf(x)
      implicit none
c
c Arguments:
c
      real*8 x
c
c Local Variables:
c
      real*8 b1,b2,b3,b4,b5,p,t,z
c
c Data:
c
      data b1,b2,b3,b4,b5,p/0.31938153d0,-0.356563782d0,1.781477937d0,
     &      -1.8212515978d0,1.330274429d0,0.2316419d0/
c
      z=x 
      if(x.lt.0d0)z=-z
      t=1d0/(1d0+p*z)
      norcdf=1d0-((0.39894228040143d0)*dexp(-0.5d0*z*z))*(b1*t+b2*t*t+
     &    b3*t**3+b4*t**4+b5*t**5)
      if(x.lt.0d0)norcdf=1d0-norcdf 
c
      return
      end 

c***** chscdf.for ***** version: 22 Jun 2008 ***************************
c Cumulative distribution function for the chi-squared distribution
c   with degrees of freedom parameter = nu, derived from
c   stat relations in the NBS applied math series and Johnson & Kotz,
c   via James Filliben's coding.
c***********************************************************************
      real*8 function chscdf(x,nu)
      implicit none
c
c Arguments:
c
      real*8 nu,x
c
c Functions:
c
      real*8 norcdf
c
c Local Variables:
c
      integer i,ievodd,imax,imin,nucut
      real*8 ai,b11,b21,b31,b32,b41,b42,b43,chi
      real*8 d1,d2,d3,danu,dcdfn,dfact,dpower,dw,pi,sd,spchi,sum
      real*8 term,term0,term1,term2,term3,term4,u,z
c 
c Data:
c
      data nucut/1000/
      data pi/3.14159265358979d0/
      data dpower/0.33333333333333d0/
      data b11/0.33333333333333d0/
      data b21/-0.02777777777778d0/
      data b31/-0.00061728395061d0/
      data b32/-13d0/
      data b41/0.00018004115226d0/
      data b42/6d0/
      data b43/17d0/
c
      if(nu.le.0)then
        write(6,*)'chscdf error: Non-positive N'
        chscdf=0d0
        return
      elseif(x.lt.0d0)then
        write(6,*)'chscdf error: Negative x'
        chscdf=0d0
        return
      endif
c
      if(x.le.0d0)then
        chscdf=0d0
        return
      endif
      sd=dsqrt(2d0*nu)
      z=(x-nu)/sd
      if(nu.lt.10.and.z.lt.-200d0)then
        chscdf=0d0
        return
      endif
      if(nu.ge.10.and.z.lt.-100d0)then
        chscdf=0d0
        return
      endif
      if(nu.lt.10.and.z.gt.200d0)then
        chscdf=1d0
        return
      endif
      if(nu.ge.10.and.z.gt.100d0)then
        chscdf=1d0
        return
      endif
c
      if(nu.lt.nucut)then
        chi=dsqrt(x) 
        ievodd=idnint(nu)-2*(idnint(nu)/2)
        if(ievodd.eq.0)then
          sum=1d0
          term=1d0
          imin=2
          imax=idnint(nu)-2
        else
          sum=0d0
          term=1.0/chi
          imin=1
          imax=nu-1
        endif
c
        if(imin.le.imax)then
          do i=imin,imax,2
            ai=dfloat(i)
            term=term*(x/ai)
            sum=sum+term
          enddo
        endif
c
        sum=sum*dexp(-x/2d0) 
        if(ievodd.ne.0)then
          sum=(dsqrt(2d0/pi))*sum
          spchi=chi
          dcdfn=norcdf(spchi) 
          sum=sum+2d0*(1d0-dcdfn)
        endif
        chscdf=1d0-sum 
        return
      elseif(nu.ge.nucut.and.x.le.nu)then
        dfact=4.5d0*nu
        u=(((x/nu)**dpower)-1d0+(1d0/dfact))*dsqrt(dfact)
        chscdf=norcdf(u)
        return
      elseif(nu.ge.nucut.and.x.gt.nu)then
        dw=dsqrt(x-nu-nu*dlog(x/nu)) 
        danu=dsqrt(2d0/nu)
        d1=dw
        d2=dw*dw
        d3=dw*d2
        term0=dw
        term1=b11*danu
        term2=b21*d1*(danu**2)
        term3=b31*(d2+b32)*(danu**3)
        term4=b41*(b42*d3+b43*d1)*(danu**4)
        u=term0+term1+term2+term3+term4
        chscdf=norcdf(u)
        return
      else
        write(6,*)'Impossible branch condition in chscdf'
        return
      endif
c
      end 

