c***** fcdfi.for ***** version: 24 Jan 2008 ****************************
c Inverse of the F-cumulative distribution function, derived from stat
c   relations in Abramowitz and Stegun's "Handbook of Mathematical
c   Functions."
c***********************************************************************
      real*8 function fcdfi(p,dm,dn)
      implicit none
c
c Arguments:
c
      real*8 dm,dn,p
c
c Function Declarations:
c
      real*8 bcdfi
c
c Local Variables:
c
      real*8 z
c
      z=bcdfi(1d0-p,dn/2d0,dm/2d0)
      fcdfi=(dn/z-dn)/dm
c
      return
      end

c***** bcdfi.for ***** version: 24 Jan 2008 ****************************
c Inverse of the beta cumulative distribution function, derived from
c   fortran code by Charles P Reeve (natl Bureau of Standards)
c***********************************************************************
      real*8 function bcdfi(p,q,r)
      implicit none
c
c Arguments:
c
      real*8 p,q,r
c
c Function Declarations:
c
      real*8 betcdf
c
c Local Variables:
c
      integer i
      real*8 a,b,c,d,e,fa,fb,fc,s,tol1,xm,u,v,w
c
      a=0d0
      b=1d0
      fa=-p
      fb=1d0-p
      fc=fb
c
      do i=1,100
        if(fb*fc.gt.0d0)then
          c=a
          fc=fa
          d=b-a
          e=d
        endif
        if(dabs(fc).lt.dabs(fb))then
          a=b
          b=c
          c=a
          fa=fb
          fb=fc
          fc=fa
        endif
c
        tol1=2d-12*dabs(b)+5d-9
        xm=(c-b)/2d0
        if(dabs(xm).le.tol1.or.fb.eq.0d0)then
          bcdfi=b
          return
        endif
c
        if(dabs(e).ge.tol1.and.dabs(fa).gt.dabs(fb))then
          s=fb/fa
          if(a.eq.c)then
            u=2d0*xm*s
            v=1d0-s
          else
            v=fa/fc
            w=fb/fc
            u=s*(2d0*xm*v*(v-w)-(b-a)*(w-1d0))
            v=(v-1d0)*(w-1d0)*(s-1d0)
          endif
c
          if(u.gt.0d0)v=-v
          u=dabs(u)
          if(2d0*u.lt.dmin1(3d0*xm*v-dabs(tol1*v),dabs(e*v)))then
            e=d
            d=u/v
          else
            d=xm
            e=d
          endif
        else
          d=xm
          e=d
        endif
c
        a=b
        fa=fb
        if(dabs(d).gt.tol1)then
          b=b+d
        else
          b=b+dsign(tol1,xm)
        endif
        fb=betcdf(b,q,r)-p
      enddo
      bcdfi=b
c
      return
      end

c***** betcdf.for ***** version: 24 Jan 2008 ***************************
c Inverse of the beta cumulative distribution function, derived from
c   fortran code by Charles P Reeve (natl Bureau of Standards)
c***********************************************************************
      real*8 function betcdf(x,p,q)
      implicit none
c
c Arguments:
c
      real*8 p,q,x
c
c Function Declarations:
c
      real*8 dgamln
c
c Local Variables:
c
      integer j
      logical ll
      real*8 dp,dq,pdfl,pq,qp,r,u,v,w,xy,yx,yxeps
c
      betcdf=0d0
      w=2d1
c
c Special cases of x
      if(x.le.0d0)return
      if(x.ge.1d0)then
        betcdf=1d0
      else
        ll=p+w.ge.(p+q+2d0*w)*x
        if(ll)then
          xy=x
          yx=1d0-xy
          pq=p
          qp=q
        else
          yx=x
          xy=1d0-yx
          qp=p
          pq=q
        endif
c
        dp=(pq-1d0)*dlog(xy)-dgamln(pq)
        dq=(qp-1d0)*dlog(yx)-dgamln(qp)
        pdfl=dgamln(pq+qp)+dp+dq
        u=dexp(pdfl)*xy/pq
        r=xy/yx
        dowhile(qp.gt.1d0)
          if(u.le.1d-8*(1d0-(pq+qp)*xy/(pq+1d0)))goto1
          betcdf=betcdf+u
          pq=pq+1d0
          qp=qp-1d0
          u=qp*r*u/pq
        enddo
        v=yx*u
        yxeps=yx*1d-8
c
        j=0
        dowhile(v.gt.yxeps.and.j.lt.5000)
          betcdf=betcdf+v
          pq=pq+1d0
          v=(pq+qp-1d0)*xy*v/pq
        enddo
1       if(.not.ll)betcdf=1d0-betcdf
      endif
c
      return
      end

c***** dgamln.for ***** version: 30 Jun 2008 ***************************
c Log of the gamma function, derived from fortran code by Charles P 
c    Reeve (Natl Bureau of Standards)
c***********************************************************************
      real*8 function dgamln(x)
      implicit none
c
c Arguments:
c
      real*8 x
c
c Local variables:
c
      integer i,n
      real*8 absacc,b1,b2,b3,b4,b5,b6,b7,b8,c,dx,q,r,xmin,xn
c
c Data:
c
      data xmin,absacc/6.894d0,1d-15/
      data c/0.918938533204672741780329736d0/
      data b1/0.833333333333333333333333333d-1/
      data b2/-0.277777777777777777777777778d-2/
      data b3/0.793650793650793650793650794d-3/
      data b4/-0.595238095238095238095238095d-3/
      data b5/0.841750841750841750841750842d-3/
      data b6/-0.191752691752691752691752692d-2/
      data b7/0.641025641025641025641025641d-2/
      data b8/-0.295506535947712418300653595d-1/
c
      x=dmax1(x,absacc)
      dx=x
      n=max0(0,idint(xmin-dx+1d0))
      xn=dx+dfloat(n)
      r=1d0/xn
      q=r*r
      dgamln=r*(b1+q*(b2+q*(b3+q*(b4+q*(b5+q*(b6+q*(b7+q*b8)))))))+c+
     &   (xn-0.5d0)*dlog(xn)-xn
c
      if(n.gt.0)then
        q=1d0
        do i=0,n-1
          q=q*(dx+dfloat(i))
        enddo
        dgamln=dgamln-dlog(q)
      endif
c
      return
      end

