c***** getdrho.for ***** version: 21 May 2018 **************************
c  For the ~ western US region, generate grids of crustal thickness H
c     and Vp/Vs ratio K at coarse (one degree) spacing by first 
c     averaging EARS estimates within pixels and then optimally 
c     interpolating to pixels with no measurements. Use the resulting
c     grids to estimate a regional-scale best estimate of Delta-rho
c     for the Moho and Int{drho/dVpVs} for use in gravmod assessment
c     of error functions from gravity modeling. Write the RMS residual
c     to a file for evaluation.
c  Implements stochastic inversion using expected values and 
c     uncertainties for Delta-rho_Moho, del-rho/del-K, and alpha
c     scaling coefficient.
c***********************************************************************
      subroutine getdrho(n0)
      implicit none
      include 'constants.inc'
c
c Arguments:
c
      integer n0
c
c Common Blocks:
c
      real*8 delg,drho,dkdrho,falpha,ffwh,ffwk,ffwt,frav,fsed
      common/DENPAR/delg,drho,dkdrho,falpha,ffwh,ffwk,ffwt,frav,fsed
c
      integer*2 etopo(4320,2160)
      real*8 elobs(ngs,ngs)
      common/ELVOBS/elobs,etopo
c
      integer n,nl
      real*8 tc_err,tc_exp,vv_err,vv_exp
      common/EXPECT/tc_err,tc_exp,vv_err,vv_exp,n,nl
c
      real*8 grav(ngi,ngj),rgdy(nti,ntj)
      common/GRAVIT/grav,rgdy
c
      integer gmaxs(ngs,ngs)
      real*8 gobs(ngs,ngs),gsed(ngs,ngs),gthm(ngs,ngs),lat(ngs,ngs)
      real*8 lon(ngs,ngs),mthm(ngs,ngs),wt(ngs,ngs),ww(ngs,ngs)
      common/GRVOBS/gobs,gsed,gthm,lat,lon,mthm,wt,ww,gmaxs
c
      character*10 site(nsta)
      integer neq(nsta),nmeas
      real*8 el(nsta),cred(nsta)
      real*8 tc(nsta),tce(nsta),vv(nsta),vve(nsta),xl(nsta),yl(nsta)
      common/INDATA/site,el,cred,tc,tce,vv,vve,xl,yl,neq,nmeas
c
      integer ntr
      complex*16 herr(ngmx,ngmx),kerr(ngmx,ngmx)
      complex*16 cgrd(ngmx,ngmx),hgrd(ngmx,ngmx),kgrd(ngmx,ngmx)
      complex*16 kmgd(ngmx,ngmx),tmgd(ngmx,ngmx),twgd(ngmx,ngmx)
      common/MODGRD/cgrd,herr,hgrd,kerr,kgrd,kmgd,tmgd,twgd,ntr
c
      integer inr(mint),nnt
      real*8 rnr(mint)
      common/NEARST/rnr,inr,nnt
c
      real*8 dkdrho_std,drho_std,falpha_std,grms
      common/STOCHA/dkdrho_std,drho_std,falpha_std,grms
c
c Function Declarations:
c
      real*8 cintg
c
c Local Variables:
c
      character line*120
      integer flg,gmi,gmj,i,ii,info,it,j
      complex*16 fwh,fwk,fwt
      real*8 gvec(5),gmat(5,5),evec(4),emat(4,4)
      real*8 gerr(ngs,ngs),hgrv(ngs,ngs),kgrv(ngs,ngs)
      real*8 ad1,ad2,dres,fdnm,fgrv,grmx,hh,hr,hx,k2d,k4,kk,kr,kx
      real*8 mobar,r,res,resh,resk,rest,rx,stmp,td,vvbar,xwt
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c Gravity model:
      ntr=ngmx
      td=2d0*pi/(dgm*dfloat(ngmx-1))
c
      mobar=0d0
      vvbar=0d0
c Plotting file for interpolated H & K:
      if(n0.eq.0)then
c       open(unit=12,file='Plots/MapHK0.xyhk',status='unknown')
        open(unit=12,file='Plots_DLS/MapHK0.xyhk',status='unknown')
        rewind(12)
c       open(unit=13,file='Plots/MapHK0.xyheke',status='unknown')
        open(unit=13,file='Plots_DLS/MapHK0.xyheke',status='unknown')
        rewind(13)
      else
c       open(unit=12,file='Plots/MapHK.xyhk',status='unknown')
        open(unit=12,file='Plots_DLS/MapHK.xyhk',status='unknown')
        rewind(12)
c       open(unit=13,file='Plots/MapHK.xyheke',status='unknown')
        open(unit=13,file='Plots_DLS/MapHK.xyheke',status='unknown')
        rewind(13)
      endif
c Parameter change file:
      if(nl.ne.0)then
        if(site(n)(6:10).eq.'     ')then
          open(unit=14,file='HK_Stacks/'//site(n)(1:5)//'.delt',
     &        status='unknown',access='append')
        elseif(site(n)(7:10).eq.'    ')then
          open(unit=14,file='HK_Stacks/'//site(n)(1:6)//'.delt',
     &        status='unknown',access='append')
        elseif(site(n)(8:10).eq.'   ')then
          open(unit=14,file='HK_Stacks/'//site(n)(1:7)//'.delt',
     &        status='unknown',access='append')
        elseif(site(n)(9:10).eq.'  ')then
          open(unit=14,file='HK_Stacks/'//site(n)(1:8)//'.delt',
     &        status='unknown',access='append')
        elseif(site(n)(10:10).eq.' ')then
          open(unit=14,file='HK_Stacks/'//site(n)(1:9)//'.delt',
     &        status='unknown',access='append')
        else
          open(unit=14,file='HK_Stacks/'//site(n)//'.delt',
     &        status='unknown',access='append')
        endif
      endif
c
c Interpolate crustal thickness and Vp/Vs to the gravity grid:
      do j=1,ngs
        do i=1,ngs
c Sort out 10 nearest points:
          call narst(lon(i,j),lat(i,j),10,0)
c Interpolation, variance estimation at grid point:
          call krige10(10,hx,hr,kx,kr)
          if(wt(i,j).ne.0d0)then
            write(12,'(f9.4,f8.4,f8.3,f7.4)')lon(i,j)/dtr,9d1-lat(i,j)/
     &          dtr,hx,kx
            write(13,'(f9.4,f8.4,2f8.3,2f7.4)')lon(i,j)/dtr,9d1-
     &          lat(i,j)/dtr,hx,hr,kx,kr
          endif
          hgrd(i,j)=dcmplx(hx,0d0)
          kgrd(i,j)=dcmplx(kx,0d0)
          herr(i,j)=dcmplx(hr,0d0)
          kerr(i,j)=dcmplx(kr,0d0)
          tmgd(i,j)=dcmplx(mthm(i,j),0d0)
          mobar=mobar+hx
          vvbar=vvbar+kx
c Mirrored grid assignment:
          hgrd(ngmx+1-i,j)=dcmplx(hx,0d0)
          hgrd(i,ngmx+1-j)=dcmplx(hx,0d0)
          hgrd(ngmx+1-i,ngmx+1-j)=dcmplx(hx,0d0)
          kgrd(ngmx+1-i,j)=dcmplx(kx,0d0)
          kgrd(i,ngmx+1-j)=dcmplx(kx,0d0)
          kgrd(ngmx+1-i,ngmx+1-j)=dcmplx(kx,0d0)
          herr(ngmx+1-i,j)=dcmplx(hr,0d0)
          herr(i,ngmx+1-j)=dcmplx(hr,0d0)
          herr(ngmx+1-i,ngmx+1-j)=dcmplx(hr,0d0)
          kerr(ngmx+1-i,j)=dcmplx(kr,0d0)
          kerr(i,ngmx+1-j)=dcmplx(kr,0d0)
          kerr(ngmx+1-i,ngmx+1-j)=dcmplx(kr,0d0)
          tmgd(ngmx+1-i,j)=dcmplx(mthm(i,j),0d0)
          tmgd(i,ngmx+1-j)=dcmplx(mthm(i,j),0d0)
          tmgd(ngmx+1-i,ngmx+1-j)=dcmplx(mthm(i,j),0d0)
        enddo
      enddo
      close(12)
      mobar=1d3*mobar/dfloat(ngs*ngs)
      vvbar=vvbar/dfloat(ngs*ngs)
c
      do j=1,ngmx
        do i=1,ngmx
c Compositional mass anomaly uses total crustal thickness; convert km to m:
          kmgd(i,j)=1d3*hgrd(i,j)
c Remove mean; convert km to m; z positive upwards:
          hgrd(i,j)=dcmplx(mobar,0d0)-1d3*hgrd(i,j)
          herr(i,j)=1d3*herr(i,j)
c Remove mean:
          kgrd(i,j)=kgrd(i,j)-dcmplx(vvbar,0d0)
c Correction of bulk crustal gravity term for thickness variations:
          cgrd(i,j)=hgrd(i,j)*kgrd(i,j)
c Crustal compositional mass is mks crustal thickness * (K - Kbar)
          kmgd(i,j)=kmgd(i,j)*kgrd(i,j)
        enddo
      enddo
c
c Calculate the gravity for unit density contrast for each grid:
      call gcfft(1,1) ! hgrd
      call gcfft(1,2) ! kgrd
      call gcfft(1,3) ! herr
      call gcfft(1,4) ! kerr
      call gcfft(1,5) ! cgrd
      call gcfft(1,6) ! tmgd
      call gcfft(1,7) ! kmgd
      do j=1,ngmx
        if(j.le.ngmx/2+1)then
          ad2=(td*dfloat(j-1))**2
        else
          ad2=(td*dfloat(ngmx+1-j))**2
        endif
        do i=1,ngmx
          if(i.le.ngmx/2+1)then
            ad1=(td*dfloat(i-1))**2
          else
            ad1=(td*dfloat(ngmx+1-i))**2
          endif
          k2d=dsqrt(ad1+ad2)
          k4=(ad1+ad2)**2
          fdnm=ro1-ro0+frav*k4/gacc
c Flexural integration of gravity; includes the Bouguer correction:
          fgrv=cgv*(cintg(k2d,drho)-ro0)
          if(k2d.ne.0d0)then
c Thermal flexure:
            fwt=-tmgd(i,j)/fdnm
c Bulk crustal lithologic flexure:
            fwk=-kmgd(i,j)/fdnm
c Moho-loading flexure:
            fwh=-hgrd(i,j)/fdnm
c           fwh=-(hgrd(i,j)-ffwt*fwt*falpha-ffwk*fwk*dkdrho)/fdnm
c           do ii=1,10
c             fwh=-(hgrd(i,j)-ffwh*fwh*drho)/fdnm
c             fwh=-(hgrd(i,j)-ffwt*fwt*falpha-ffwk*fwk*dkdrho-ffwh*fwh*
c    &            drho)/fdnm
c           enddo
c Gravity anomaly for unit Moho density contrast:
c           hgrd(i,j)=hgrd(i,j)*cgv*dexp(-k2d*mobar)+fgrv*fwh*ffwh
            hgrd(i,j)=hgrd(i,j)*cgv*dexp(-k2d*mobar)*(1d0-(1d0/(1d0+
     &          frav*k4/gacc)))
c Gravity error for unit Moho density contrast:
            herr(i,j)=herr(i,j)*cgv*dexp(-k2d*mobar)
c Gravity anomaly for unit Pratt density variation:
            kgrd(i,j)=(kgrd(i,j)*cgv*(1d0-dexp(-k2d*mobar))/k2d
c Correction for thickness of bulk crust:
c    &          -cgrd(i,j)*cgv*dexp(-k2d*mobar)+fgrv*fwk*ffwk
     &          -cgrd(i,j)*cgv*dexp(-k2d*mobar))*(1d0-(1d0/(1d0+
     &          frav*k4/gacc)))
c Gravity error for unit Pratt density variation:
            kerr(i,j)=kerr(i,j)*cgv*(1d0-dexp(-k2d*mobar))/k2d
            tmgd(i,j)=fgrv*fwt*ffwt
            cgrd(i,j)=fwh
            kmgd(i,j)=fwk
            twgd(i,j)=fwt
          else
            hgrd(i,j)=dcmplx(0d0,0d0)
            herr(i,j)=herr(i,j)*cgv
            kgrd(i,j)=dcmplx(0d0,0d0)
            kerr(i,j)=kerr(i,j)*cgv
            cgrd(i,j)=dcmplx(0d0,0d0)
            tmgd(i,j)=dcmplx(0d0,0d0)
            kmgd(i,j)=dcmplx(0d0,0d0)
            twgd(i,j)=dcmplx(0d0,0d0)
          endif
        enddo
      enddo
      call gcfft(-1,1) ! hgrd
      call gcfft(-1,2) ! kgrd
      call gcfft(-1,3) ! herr
      call gcfft(-1,4) ! kerr
      call gcfft(-1,5) ! cgrd = fwh
      call gcfft(-1,6) ! tmgd
      call gcfft(-1,7) ! kmgd = fwk
      call gcfft(-1,8) ! twgd = fwt
c
c Calculate regression coefficients to the observed topography:
      do j=1,4
        do i=1,4
          emat(i,j)=0d0
        enddo
        evec(j)=0d0
      enddo
      xwt=0d0
      do j=1,ngs
        do i=1,ngs
c NOTE: This is not kosher, because we are treating these weights as though
c     they are data uncertainties (i.e., gravity in the d vector) when they
c     are in fact uncertainties in the kernel matrix values (H & K).
          gerr(i,j)=ww(i,j)/((drho*dreal(herr(i,j)))**2+(dkdrho*
     &        dreal(kerr(i,j)))**2)
c This is not a problem for WLS, but it creates problems for SI. At some point
c     the weight matrix needs to be treated in a less ad-hoc manner.
          xwt=xwt+gerr(i,j)
          emat(1,1)=emat(1,1)+gerr(i,j)
          emat(2,1)=emat(2,1)+gerr(i,j)*dreal(cgrd(i,j))
          emat(3,1)=emat(3,1)+gerr(i,j)*dreal(kmgd(i,j))
          emat(4,1)=emat(4,1)+gerr(i,j)*dreal(twgd(i,j))
          emat(2,2)=emat(2,2)+gerr(i,j)*dreal(cgrd(i,j)*cgrd(i,j))
          emat(3,2)=emat(3,2)+gerr(i,j)*dreal(cgrd(i,j)*kmgd(i,j))
          emat(4,2)=emat(4,2)+gerr(i,j)*dreal(cgrd(i,j)*twgd(i,j))
          emat(3,3)=emat(3,3)+gerr(i,j)*dreal(kmgd(i,j)*kmgd(i,j))
          emat(4,3)=emat(4,3)+gerr(i,j)*dreal(kmgd(i,j)*twgd(i,j))
          emat(4,4)=emat(4,4)+gerr(i,j)*dreal(twgd(i,j)*twgd(i,j))
          dres=elobs(i,j)-dele_bar-drho_bar*dreal(cgrd(i,j))-dkdrho_bar*
     &        dreal(kmgd(i,j))-falpha_bar*dreal(twgd(i,j))
          evec(1)=evec(1)+gerr(i,j)*dres
          evec(2)=evec(2)+gerr(i,j)*dreal(cgrd(i,j))*dres
          evec(3)=evec(3)+gerr(i,j)*dreal(kmgd(i,j))*dres
          evec(4)=evec(4)+gerr(i,j)*dreal(twgd(i,j))*dres
        enddo
      enddo
      emat(1,1)=emat(1,1)+(1d0/(dele_std*dele_std))
      emat(2,2)=emat(2,2)+(1d0/(drho_std*drho_std))
      emat(3,3)=emat(3,3)+(1d0/(dkdrho_std*dkdrho_std))
      emat(4,4)=emat(4,4)+(1d0/(falpha_std*falpha_std))
      call dposv('l',4,1,emat,4,evec,4,info)
c
      evec(1)=evec(1)+dele_bar
      ffwh=evec(2)+drho_bar
      ffwk=evec(3)+dkdrho_bar
      ffwt=evec(4)+falpha_bar
c
      res=0d0
      resh=0d0
      resk=0d0
      rest=0d0
      do j=1,ngs
        do i=1,ngs
          rx=elobs(i,j)-evec(1)-ffwh*dreal(cgrd(i,j))-ffwk*
     &        dreal(kmgd(i,j))-ffwt*dreal(twgd(i,j))
          res=res+gerr(i,j)*rx*rx
          rx=elobs(i,j)-evec(1)-ffwh*dreal(cgrd(i,j))
          resh=resh+gerr(i,j)*rx*rx
          rx=elobs(i,j)-evec(1)-ffwk*dreal(kmgd(i,j))
          resk=resk+gerr(i,j)*rx*rx
          rx=elobs(i,j)-evec(1)-ffwt*dreal(twgd(i,j))
          rest=rest+gerr(i,j)*rx*rx
        enddo
      enddo
      res=dsqrt(res/xwt)
      resh=dsqrt(resh/xwt)
      resk=dsqrt(resk/xwt)
      rest=dsqrt(rest/xwt)
c
c     open(unit=43,file='Plots/eresidual.dat',status='unknown')
      open(unit=43,file='Plots_DLS/eresidual.dat',status='unknown')
      if(nl.ne.0)then
        read(43,'(a70)')line
        write(14,'(a70)')line
        rewind(43)
        write(14,'(i6,f8.4,f8.3,f8.2,5f8.4)')n0,evec(1)/1d3,ffwh,ffwk,
     &      ffwt,resh/1d3,resk/1d3,rest/1d3,res/1d3
      endif
      write(43,'(i6,f8.4,f8.3,f8.2,5f8.4)')n0,evec(1)/1d3,ffwh,ffwk,
     &    ffwt,resh/1d3,resk/1d3,rest/1d3,res/1d3
      close(43)
c     open(unit=44,file='Plots/eresiduals.dat',status='unknown',
      open(unit=44,file='Plots_DLS/eresiduals.dat',status='unknown',
     &    access='append')
      write(44,'(i6,f8.4,f8.3,f8.2,5f8.4)')n0,evec(1)/1d3,ffwh,ffwk,
     &    ffwt,resh/1d3,resk/1d3,rest/1d3,res/1d3
      close(44)
      write(6,*)
      write(6,'(i6,f8.4,f8.3,f8.2,5f8.4)')n0,evec(1)/1d3,ffwh,ffwk,ffwt,
     &    resh/1d3,resk/1d3,rest/1d3,res/1d3
c
c Save the best-fit topography model to a file:
      if(n0.eq.0)then
c       open(unit=12,file='Plots/MapTopo0.xyohktm',status='unknown')
        open(unit=12,file='Plots_DLS/MapTopo0.xyohktm',status='unknown')
        rewind(12)
      else
c       open(unit=12,file='Plots/MapTopo.xyohktm',status='unknown')
        open(unit=12,file='Plots_DLS/MapTopo.xyohktm',status='unknown')
        rewind(12)
      endif
      do j=1,ngs
        do i=1,ngs
c         if(wt(i,j).ne.0d0)then
c           write(12,'(f9.4,f8.4,5f9.1)')lon(i,j)/dtr,9d1-lat(i,j)/dtr,
c    &          elobs(i,j),evec(1)+evec(2)*dreal(cgrd(i,j)),evec(1)+
c    &          evec(3)*dreal(kmgd(i,j)),evec(1)+evec(4)*falpha*
c    &          dreal(twgd(i,j)),evec(1)+evec(2)*dreal(cgrd(i,j))+
c    &          evec(3)*dreal(kmgd(i,j))+evec(4)*falpha*dreal(twgd(i,j))
            write(12,'(f9.4,f8.4,5f9.1)')lon(i,j)/dtr,9d1-lat(i,j)/dtr,
     &          elobs(i,j),evec(1)+dreal(cgrd(i,j)),evec(1)+
     &          dreal(kmgd(i,j)),evec(1)+falpha*
     &          dreal(twgd(i,j)),evec(1)+dreal(cgrd(i,j))+
     &          dreal(kmgd(i,j))+falpha*dreal(twgd(i,j))
c         endif
        enddo
      enddo
      close(12)
c
c Calculate regression coefficients to the observed gravity:
      do j=1,5
        do i=1,5
          gmat(i,j)=0d0
        enddo
        gvec(j)=0d0
      enddo
      do j=1,ngs
        do i=1,ngs
          gmat(1,1)=gmat(1,1)+gerr(i,j)
          gmat(2,1)=gmat(2,1)+gerr(i,j)*dreal(hgrd(i,j))
          gmat(3,1)=gmat(3,1)+gerr(i,j)*dreal(kgrd(i,j))
          gmat(4,1)=gmat(4,1)+gerr(i,j)*gthm(i,j)
          gmat(5,1)=gmat(5,1)+gerr(i,j)*gsed(i,j)
          gmat(2,2)=gmat(2,2)+gerr(i,j)*dreal(hgrd(i,j)*hgrd(i,j))
          gmat(3,2)=gmat(3,2)+gerr(i,j)*dreal(hgrd(i,j)*kgrd(i,j))
          gmat(4,2)=gmat(4,2)+gerr(i,j)*dreal(hgrd(i,j))*gthm(i,j)
          gmat(5,2)=gmat(5,2)+gerr(i,j)*dreal(hgrd(i,j))*gsed(i,j)
          gmat(3,3)=gmat(3,3)+gerr(i,j)*dreal(kgrd(i,j)*kgrd(i,j))
          gmat(4,3)=gmat(4,3)+gerr(i,j)*dreal(kgrd(i,j))*gthm(i,j)
          gmat(5,3)=gmat(5,3)+gerr(i,j)*dreal(kgrd(i,j))*gsed(i,j)
          gmat(4,4)=gmat(4,4)+gerr(i,j)*gthm(i,j)*gthm(i,j)
          gmat(5,4)=gmat(5,4)+gerr(i,j)*gthm(i,j)*gsed(i,j)
          gmat(5,5)=gmat(5,5)+gerr(i,j)*gsed(i,j)*gsed(i,j)
          dres=gobs(i,j)-delg_bar-drho_bar*dreal(hgrd(i,j))-dkdrho_bar*
     &        dreal(kgrd(i,j))-falpha_bar*gthm(i,j)-fsed_bar*gsed(i,j)
          gvec(1)=gvec(1)+gerr(i,j)*dres
          gvec(2)=gvec(2)+gerr(i,j)*dreal(hgrd(i,j))*dres
          gvec(3)=gvec(3)+gerr(i,j)*dreal(kgrd(i,j))*dres
          gvec(4)=gvec(4)+gerr(i,j)*gthm(i,j)*dres
          gvec(5)=gvec(5)+gerr(i,j)*gsed(i,j)*dres
        enddo
      enddo
      gmat(1,1)=gmat(1,1)+(1d0/(delg_std*delg_std))
      gmat(2,2)=gmat(2,2)+(1d0/(drho_std*drho_std))
      gmat(3,3)=gmat(3,3)+(1d0/(dkdrho_std*dkdrho_std))
      gmat(4,4)=gmat(4,4)+(1d0/(falpha_std*falpha_std))
      gmat(5,5)=gmat(5,5)+(1d0/(fsed_std*fsed_std))
      call dposv('l',5,1,gmat,5,gvec,5,info)
      gvec(1)=gvec(1)+delg_bar
      gvec(2)=gvec(2)+drho_bar
      gvec(3)=gvec(3)+dkdrho_bar
      gvec(4)=gvec(4)+falpha_bar
      gvec(5)=gvec(5)+fsed_bar
      do j=1,ngs
        do i=1,ngs
          hgrv(i,j)=gvec(2)*dreal(hgrd(i,j))
          kgrv(i,j)=gvec(3)*dreal(kgrd(i,j))
        enddo
      enddo
      res=0d0
      resh=0d0
      resk=0d0
      rest=0d0
      grmx=0d0
      do j=1,ngs
        do i=1,ngs
          rx=gobs(i,j)-gvec(1)-hgrv(i,j)-kgrv(i,j)-
     &        gvec(4)*gthm(i,j)-gvec(5)*gsed(i,j)
          if(dabs(rx).gt.grmx.and.gmaxs(i,j).lt.2)then
            grmx=dabs(rx)
            gmi=i
            gmj=j
          endif
          res=res+gerr(i,j)*rx*rx
          rx=gobs(i,j)-gvec(1)-hgrv(i,j)
          resh=resh+gerr(i,j)*rx*rx
          rx=gobs(i,j)-gvec(1)-kgrv(i,j)
          resk=resk+gerr(i,j)*rx*rx
          rx=gobs(i,j)-gvec(1)-gvec(4)*gthm(i,j)
          rest=rest+gerr(i,j)*rx*rx
        enddo
      enddo
      res=dsqrt(res/xwt)
      resh=dsqrt(resh/xwt)
      resk=dsqrt(resk/xwt)
      rest=dsqrt(rest/xwt)
c     open(unit=43,file='Plots/gresidual.dat',status='unknown')
      open(unit=43,file='Plots_DLS/gresidual.dat',status='unknown')
      if(nl.ne.0)then
        read(43,'(a70)')line
        write(14,'(a70)')line
        rewind(43)
        write(14,'(i6,f8.2,f8.3,f8.2,5f8.4)')n0,gvec(1),gvec(2),gvec(3),
     &      gvec(4),resh,resk,rest,res
      endif
      write(43,'(i6,f8.2,f8.3,f8.2,5f8.4)')n0,gvec(1),gvec(2),gvec(3),
     &    gvec(4),resh,resk,rest,res
      close(43)
c     open(unit=44,file='Plots/gresiduals.dat',status='unknown',
      open(unit=44,file='Plots_DLS/gresiduals.dat',status='unknown',
     &    access='append')
      write(44,'(i6,f8.2,f8.3,f8.2,5f8.4)')n0,gvec(1),gvec(2),gvec(3),
     &    gvec(4),resh,resk,rest,res
      close(44)
c     open(unit=45,file='Plots/resmax.dat',status='unknown',
      open(unit=45,file='Plots_DLS/resmax.dat',status='unknown',
     &    access='append')
      write(45,'(f9.4,f8.4,f7.1)')lon(gmi,gmj)/dtr,9d1-lat(gmi,gmj)/dtr,
     &    grmx
      close(45)
      write(6,*)
      write(6,'(i6,f8.2,f8.3,f8.2,6f8.4)')n0,gvec(1),gvec(2),gvec(3),
     &    gvec(4),gvec(5),resh,resk,rest,res
      write(6,'(a14,f9.4,f8.4,f7.1)')'Max grav res: ',lon(gmi,gmj)/dtr,
     &    9d1-lat(gmi,gmj)/dtr,grmx
      call narst(lon(gmi,gmj),lat(gmi,gmj),150,0)
      gmaxs(gmi,gmj)=gmaxs(gmi,gmj)+1
c
      delg=gvec(1)
      drho=gvec(2)
      dkdrho=gvec(3)
      falpha=gvec(4)
      fsed=gvec(5)
c
c If gravity has no predictive power, loosen the stochastic parameter variance:
c     if(resk.gt.grms.and.dkdrho.gt.dkdrho_bar/2d0)then
c       dkdrho_std=dkdrho_std*1.001d0
c       write(6,*)
c       write(6,'(a39,f8.4)')'drho/dK stochastic parameter modified: ',
c    &      dkdrho_std
c     endif
c     if(res.gt.grms)then
c       drho_std=drho_std*1.001d0
c       falpha_std=falpha_std*1.001d0
c       write(6,*)
c       write(6,'(a32,2f8.4)')'Stochastic parameters modified: ',
c    &      drho_std,falpha_std
c     endif
c     open(unit=21,file='Plots_DLS/stdincs.dat',status='unknown')
c     rewind(21)
c     write(21,'(3f9.5)')drho_std,dkdrho_std,falpha_std
c     close(21)
c
c Save the best-fit gravity model to a file:
      if(n0.eq.0)then
c       open(unit=12,file='Plots/MapGrav0.xyohktm',status='unknown')
        open(unit=12,file='Plots_DLS/MapGrav0.xyohktm',status='unknown')
        rewind(12)
      else
c       open(unit=12,file='Plots/MapGrav.xyohktm',status='unknown')
        open(unit=12,file='Plots_DLS/MapGrav.xyohktm',status='unknown')
        rewind(12)
      endif
      do j=1,ngs
        do i=1,ngs
c         if(wt(i,j).ne.0d0)then
            write(12,'(f9.4,f8.4,5f7.1)')lon(i,j)/dtr,9d1-lat(i,j)/dtr,
     &          gobs(i,j),delg+hgrv(i,j),delg+kgrv(i,j),delg+
     &          falpha*gthm(i,j),delg+hgrv(i,j)+kgrv(i,j)+falpha*
     &          gthm(i,j)+fsed*gsed(i,j)
c         endif
        enddo
      enddo
      close(12)
      close(14)
c
      return
      end

