c***** gravmod.for ***** version: 21 May 2018 **************************
c  For stack range of H & K, optimally interpolate current estimates 
c     H & K from their values at other sites plus an assigned value
c     at the update site to a gravity grid mesh.  The assigned value
c     at the update site is iterated upon in a grid search fashion
c     to evaluate best-fit and confidence intervals from misfit
c     to the observed gravity field.  Uncertainty estimates are used 
c     subsequently to modify the error function for that site. 
c***********************************************************************
      subroutine gravmod
      implicit none
      include 'constants.inc'
c
c Common Blocks:
c
      real*8 delg,drho,dkdrho,falpha,ffwh,ffwk,ffwt,frav,fsed
      common/DENPAR/delg,drho,dkdrho,falpha,ffwh,ffwk,ffwt,frav,fsed
c
      integer n,nl
      real*8 tc_err,tc_exp,vv_err,vv_exp
      common/EXPECT/tc_err,tc_exp,vv_err,vv_exp,n,nl
c
      real*8 grav(ngi,ngj),rgdy(nti,ntj)
      common/GRAVIT/grav,rgdy
c
      integer nhh,nkk
      real*8 gres(40,40),prb(127)
      real*8 grmn,hhmn,hhmx,hmn,kkmn,kkmx,kmn,wtn
      common/GRAVWT/gres,prb,grmn,hhmn,hhmx,hmn,kkmn,kkmx,kmn,wtn,nhh,
     &    nkk
c
      integer gmaxs(ngs,ngs)
      real*8 gobs(ngs,ngs),gsed(ngs,ngs),gthm(ngs,ngs),lat(ngs,ngs)
      real*8 lon(ngs,ngs),mthm(ngs,ngs),wt(ngs,ngs),ww(ngs,ngs)
      common/GRVOBS/gobs,gsed,gthm,lat,lon,mthm,wt,ww,gmaxs
c
      character*10 site(nsta)
      integer neq(nsta),nmeas
      real*8 el(nsta),cred(nsta)
      real*8 tc(nsta),tce(nsta),vv(nsta),vve(nsta),xl(nsta),yl(nsta)
      common/INDATA/site,el,cred,tc,tce,vv,vve,xl,yl,neq,nmeas
c
      integer ntr
      complex*16 herr(ngmx,ngmx),kerr(ngmx,ngmx)
      complex*16 cgrd(ngmx,ngmx),hgrd(ngmx,ngmx),kgrd(ngmx,ngmx)
      complex*16 kmgd(ngmx,ngmx),tmgd(ngmx,ngmx),twgd(ngmx,ngmx)
      common/MODGRD/cgrd,herr,hgrd,kerr,kgrd,kmgd,tmgd,twgd,ntr
c
      integer inr(mint),nnt
      real*8 rnr(mint)
      common/NEARST/rnr,inr,nnt
c
      real*8 dcy(nsta),dsy(nsta)
      common/STORCS/dcy,dsy
c
c Function Declarations:
c
      real*8 cintg,fcdfi
c
c Local Variables:
c
      integer ists(300),ir(5),jr(5)
      integer i,i1,ii,info,it,j,j1,jj,k,l,mid,nsts
      complex*16 fwh,fwk,fwt
      real*8 cgrv(ngr,ngr),gosg(ngr,ngr),hgrv(ngr,ngr),kgrv(ngr,ngr)
      real*8 ltr(ngr,ngr),lnr(ngr,ngr),ogrv(ngr,ngr),wtr(ngr,ngr)
      real*8 gerr(ngr,ngr),tgrv(ngr,ngr),sgrv(ngr,ngr),rr(5),zs(5)
      real*8 gmsm(ngr,ngr),gmss(ngr,ngr)
      real*8 ad1,ad2,adr,ccs,cl,dcp,df,dsp,fdnm,fgrv,fr
      real*8 gbar,gmd,grms,hh,hr,hx,k2d,k4,kk,kr,kx,lm0,lo,lt,lh
      real*8 mdn,mobar,parm,parn,ph,ph1,r,r1,r2,r3,r4,rho,res,rx
      real*8 ssbar,ssmx,stmp
      real*8 tck,td,th,ttbar,ttmbr,ttn,vvbar,vvk,x,xwt,y
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c Regrid and (Lambert azimuthal equal-area) project gravity to a 
c    20-km spaced, 640x640 km grid centered at the site n:
      ph1=pi/2d0-yl(n)
      dcp=dcos(ph1)
      dsp=dsin(ph1)
      lm0=xl(n)
      mid=(ngr+1)/2
      adr=drho
c
c Local flexural rigidity:
      i=idnint(4d0*(lm0/dtr+1.5d2))+1
      j=idnint(4d0*(ph1/dtr-11d0))+1
      if(i.ge.1.and.i.le.nti.and.j.ge.1.and.j.le.ntj)then
        fr=rgdy(i,j)
      else
        fr=frav
      endif
c
      wtn=0d0
      gbar=0d0
      grms=0d0
c
      do j=1,ngr
        y=dgr*dfloat(j-mid)
        do i=1,ngr
          x=dgr*dfloat(i-mid)
          rho=dsqrt(x*x+y*y)
          cl=2d0*dasin(rho/(2d0*A))
          lo=lm0+datan2(x*dsin(cl),rho*dcp*dcos(cl)-y*dsp*dsin(cl))
          lt=dasin(dcos(cl)*dsp+y*dsin(cl)*dcp/rho)
          if(i.eq.mid.and.j.eq.mid)lt=ph1
          lh=pi/2d0-lt
          lnr(i,j)=lo
          ltr(i,j)=lh
          ii=idint(30d0*(lo/dtr+144d0))+1
          i1=min(ngi-1,max(1,ii))
          jj=idint(30d0*(64d0-lt/dtr))+1
          j1=min(ngj-1,max(1,jj))
          if(grav(i1,j1).eq.-9999d0.or.grav(i1,j1+1).eq.-9999d0.or.grav
     &        (i1+1,j1).eq.-9999d0.or.grav(i1+1,j1+1).eq.-9999d0.or.ii.
     &        lt.1.or.ii.gt.ngi.or.jj.lt.1.or.jj.gt.ngj)then
            gosg(i,j)=-9999d0
            wtr(i,j)=0d0
          else
            ph=dtr*(dfloat(i1-1)/30d0-144d0)
            th=dtr*(dfloat(j1-1)/30d0+26d0)
            ccs=dcos(lh)*dcos(th)+dsin(lh)*dsin(th)*dcos(lo-ph)
            ccs=dmax1(dmin1(ccs,1d0),-1d0)
            r1=A*dacos(ccs)
            ph=dtr*(dfloat(i1)/30d0-144d0)
            ccs=dcos(lh)*dcos(th)+dsin(lh)*dsin(th)*dcos(lo-ph)
            ccs=dmax1(dmin1(ccs,1d0),-1d0)
            r2=A*dacos(ccs)
            th=dtr*(dfloat(j1)/30d0+26d0)
            ccs=dcos(lh)*dcos(th)+dsin(lh)*dsin(th)*dcos(lo-ph)
            ccs=dmax1(dmin1(ccs,1d0),-1d0)
            r3=A*dacos(ccs)
            ph=dtr*(dfloat(i1-1)/30d0-144d0)
            ccs=dcos(lh)*dcos(th)+dsin(lh)*dsin(th)*dcos(lo-ph)
            ccs=dmax1(dmin1(ccs,1d0),-1d0)
            r4=A*dacos(ccs)
            gosg(i,j)=(r2*r3*r4*grav(i1,j1)+r1*r3*r4*grav(i1+1,j1)+
     &          r1*r2*r4*grav(i1+1,j1+1)+r1*r2*r3*grav(i1,j1+1))/
     &          (r2*r3*r4+r1*r3*r4+r1*r2*r4+r1*r2*r3)
            wtr(i,j)=1d0
            wtn=wtn+1d0
            gbar=gbar+gosg(i,j)
            grms=grms+gosg(i,j)*gosg(i,j)
          endif
          tgrv(i,j)=-9999d0
          tmgd(i,j)=dcmplx(0d0,0d0)
          gmsm(i,j)=0d0
          gmss(i,j)=0d0
          do k=1,4
            rr(k)=1d10
            ir(k)=0
            jr(k)=0
          enddo
c Re-interpolate sedimentary gravity, thermal gravity and mass:
          do jj=1,ngs
            do ii=1,ngs
              ph=lon(ii,jj)
              th=lat(ii,jj)
              ccs=dcos(lh)*dcos(th)+dsin(lh)*dsin(th)*dcos(lo-ph)
              ccs=dmax1(dmin1(ccs,1d0),-1d0)
              r=A*dacos(ccs)
              do k=4,1,-1
                if(r.le.rr(k))then
                  rr(k+1)=rr(k)
                  ir(k+1)=ir(k)
                  jr(k+1)=jr(k)
                  rr(k)=r
                  ir(k)=ii
                  jr(k)=jj
                endif
              enddo
            enddo
          enddo
          sgrv(i,j)=(rr(2)*rr(3)*rr(4)*gsed(ir(1),jr(1))+rr(1)*rr(3)*
     &        rr(4)*gsed(ir(2),jr(2))+rr(1)*rr(2)*rr(4)*
     &        gsed(ir(3),jr(3))+rr(1)*rr(2)*rr(3)*gsed(ir(4),jr(4)))/
     &        (rr(2)*rr(3)*rr(4)+rr(1)*rr(3)*rr(4)+rr(1)*rr(2)*rr(4)+
     &        rr(1)*rr(2)*rr(3))
          sgrv(i,j)=sgrv(i,j)*fsed
          tgrv(i,j)=(rr(2)*rr(3)*rr(4)*gthm(ir(1),jr(1))+rr(1)*rr(3)*
     &        rr(4)*gthm(ir(2),jr(2))+rr(1)*rr(2)*rr(4)*
     &        gthm(ir(3),jr(3))+rr(1)*rr(2)*rr(3)*gthm(ir(4),jr(4)))/
     &        (rr(2)*rr(3)*rr(4)+rr(1)*rr(3)*rr(4)+rr(1)*rr(2)*rr(4)+
     &        rr(1)*rr(2)*rr(3))
          tgrv(i,j)=tgrv(i,j)*falpha
          tmgd(i,j)=dcmplx((rr(2)*rr(3)*rr(4)*mthm(ir(1),jr(1))+rr(1)*
     &        rr(3)*rr(4)*mthm(ir(2),jr(2))+rr(1)*rr(2)*rr(4)*
     &        mthm(ir(3),jr(3))+rr(1)*rr(2)*rr(3)*mthm(ir(4),jr(4)))/
     &        (rr(2)*rr(3)*rr(4)+rr(1)*rr(3)*rr(4)+rr(1)*rr(2)*rr(4)+
     &        rr(1)*rr(2)*rr(3)),0d0)
          tmgd(i,j)=tmgd(i,j)*falpha
        enddo
      enddo
      if(wtn.ge.25d0)then
        grms=dsqrt((wtn*grms-gbar*gbar)/(wtn*(wtn-1d0)))
        gbar=gbar/wtn
      endif
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c Open and read the EARS HK stack-file to define limits for grid search;
c Open the output file for plotting gravity residual used to re-weight:
      if(site(n)(6:10).eq.'     ')then
        open(unit=11,file='HK_Stacks/'//site(n)(1:5)//'.xyz',
     &      status='old')
        open(unit=12,file='HK_Stacks/'//site(n)(1:5)//'_gres.xyz',
     &      status='unknown')
      elseif(site(n)(7:10).eq.'    ')then
        open(unit=11,file='HK_Stacks/'//site(n)(1:6)//'.xyz',
     &      status='old')
        open(unit=12,file='HK_Stacks/'//site(n)(1:6)//'_gres.xyz',
     &      status='unknown')
      elseif(site(n)(8:10).eq.'   ')then
        open(unit=11,file='HK_Stacks/'//site(n)(1:7)//'.xyz',
     &      status='old')
        open(unit=12,file='HK_Stacks/'//site(n)(1:7)//'_gres.xyz',
     &      status='unknown')
      elseif(site(n)(9:10).eq.'  ')then
        open(unit=11,file='HK_Stacks/'//site(n)(1:8)//'.xyz',
     &      status='old')
        open(unit=12,file='HK_Stacks/'//site(n)(1:8)//'_gres.xyz',
     &      status='unknown')
      elseif(site(n)(10:10).eq.' ')then
        open(unit=11,file='HK_Stacks/'//site(n)(1:9)//'.xyz',
     &      status='old')
        open(unit=12,file='HK_Stacks/'//site(n)(1:9)//'_gres.xyz',
     &      status='unknown')
      else
        open(unit=11,file='HK_Stacks/'//site(n)//'.xyz',
     &      status='old')
        open(unit=12,file='HK_Stacks/'//site(n)//'_gres.xyz',
     &      status='unknown')
      endif
      rewind(11)
      rewind(12)
      write(12,*)0d0,0d0,gbar,0d0,0d0,grms
c
c Read in the stack data:
      kkmn=1d20
      kkmx=-1d20
      hhmn=1d20
      hhmx=-1d20
      read(11,*)
1     read(11,*,end=2)kk,hh,stmp
        hh=hh-el(n)/1d3
        kkmn=dmin1(kk,kkmn)
        kkmx=dmax1(kk,kkmx)
        hhmn=dmin1(hh,hhmn)
        hhmx=dmax1(hh,kkmx)
      goto1
2     close(11)
      nkk=idnint((kkmx-kkmn)/2.5d-2)+1
      if(kkmn+dfloat(nkk-1)*2.5d-2.lt.kkmx)nkk=nkk+1
      nhh=idnint((hhmx-hhmn)/2.5d0)+1
      if(hhmn+dfloat(nhh-1)*2.5d0.lt.hhmx)nhh=nhh+1
      if(wtn.lt.25d0)return                   ! If no gravity available
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c Gravity modeling: Original (pre-update) grav map for plotting:
      ntr=ngm
      td=2d0*pi/(dgr*dfloat(ngm-1))
      mobar=0d0
      vvbar=0d0
      ssbar=0d0
      ttbar=0d0
      ttmbr=0d0
      do i=1,300
        ists(i)=0
      enddo
      ists(1)=n
      nsts=1
c
      ttn=0d0
c Interpolate crustal thickness and Vp/Vs to the gravity grid:
      do j=1,ngr
        do i=1,ngr
c Sort out 10 nearest points:
          call ignarst(lnr(i,j),ltr(i,j),4d4,10,0)
c Test for duplicate radii:
          ccs=dcos(ltr(i,j))*dcy(n)+dsin(ltr(i,j))*dsy(n)*dcos(lnr(i,j)
     &       -xl(n))
          ccs=dmax1(dmin1(ccs,1d0),-1d0)
          r=A*dacos(ccs)
          it=0
          do ii=1,10
            if(dabs(r-rnr(ii)).lt.1d0)it=ii
          enddo
          if(it.eq.0)then
c Set the tenth to point n:
            inr(10)=n
            rnr(10)=r
          else
c Replace duplicate radius in interpolation:
            inr(it)=n
            rnr(it)=r
          endif
          if(rnr(1).gt.7d4.and.r.gt.7d4.and.wtr(i,j).eq.1d0)then
            wtr(i,j)=0d0
            wtn=wtn-1d0
            gbar=gbar-gosg(i,j)
            grms=grms-gosg(i,j)*gosg(i,j)
          endif
c Save site list for plotfile:
          do ii=1,10
            if(inr(ii).lt.ists(1))then
              do it=nsts+1,2,-1
                ists(it)=ists(it-1)
              enddo
              ists(1)=inr(ii)
              nsts=nsts+1
            elseif(inr(ii).gt.ists(nsts))then
              nsts=nsts+1
              ists(nsts)=inr(ii)
            else
              do it=1,nsts-1
                if(inr(ii).gt.ists(it).and.inr(ii).lt.
     &              ists(it+1))then
                  do i1=nsts+1,it+1,-1
                    ists(i1)=ists(i1-1)
                  enddo
                  ists(it+1)=inr(ii)
                  nsts=nsts+1
                endif
              enddo
            endif
          enddo
c Interpolation, variance estimation at grid point:
          call krige10(10,hx,hr,kx,kr)
          hgrd(i,j)=dcmplx(hx,0d0)
          kgrd(i,j)=dcmplx(kx,0d0)
          herr(i,j)=dcmplx(hr,0d0)
          kerr(i,j)=dcmplx(kr,0d0)
          mobar=mobar+hx
          vvbar=vvbar+kx
          if(tgrv(i,j).ne.-9999d0)then
            ttbar=ttbar+tgrv(i,j)
            ttmbr=ttmbr+dreal(tmgd(i,j))
            ttn=ttn+1d0
          endif
          ssbar=ssbar+sgrv(i,j)
c Mirrored grid assignment:
          hgrd(ngm+1-i,j)=dcmplx(hx,0d0)
          hgrd(i,ngm+1-j)=dcmplx(hx,0d0)
          hgrd(ngm+1-i,ngm+1-j)=dcmplx(hx,0d0)
          kgrd(ngm+1-i,j)=dcmplx(kx,0d0)
          kgrd(i,ngm+1-j)=dcmplx(kx,0d0)
          kgrd(ngm+1-i,ngm+1-j)=dcmplx(kx,0d0)
          herr(ngm+1-i,j)=dcmplx(hr,0d0)
          herr(i,ngm+1-j)=dcmplx(hr,0d0)
          herr(ngm+1-i,ngm+1-j)=dcmplx(hr,0d0)
          kerr(ngm+1-i,j)=dcmplx(kr,0d0)
          kerr(i,ngm+1-j)=dcmplx(kr,0d0)
          kerr(ngm+1-i,ngm+1-j)=dcmplx(kr,0d0)
          tmgd(ngm+1-i,j)=tmgd(i,j)
          tmgd(i,ngm+1-j)=tmgd(i,j)
          tmgd(ngm+1-i,ngm+1-j)=tmgd(i,j)
        enddo
      enddo
      mobar=1d3*mobar/dfloat(ngr*ngr)
      vvbar=vvbar/dfloat(ngr*ngr)
      if(ttn.ne.0d0)then
        ttbar=ttbar/ttn
        ttmbr=ttmbr/ttn
      else
        ttbar=-9999d0
        ttmbr=-9999d0
      endif
      ssbar=ssbar/dfloat(ngr*ngr)
c
      do j=1,ngm
        do i=1,ngm
c Compositional mass anomaly uses total crustal thickness; convert km to m:
          kmgd(i,j)=1d3*hgrd(i,j)
c Remove mean; convert km to m; z positive upwards:
          hgrd(i,j)=dcmplx(mobar,0d0)-1d3*hgrd(i,j)
          herr(i,j)=1d3*herr(i,j)
c Remove mean:
          kgrd(i,j)=kgrd(i,j)-dcmplx(vvbar,0d0)
          if(ttmbr.ne.-9999d0)tmgd(i,j)=tmgd(i,j)-dcmplx(ttmbr,0d0)
c Correction of bulk crustal gravity term for thickness variations:
          cgrd(i,j)=hgrd(i,j)*kgrd(i,j)
c Crustal compositional mass is mks crustal thickness * (K - Kbar)
          kmgd(i,j)=kmgd(i,j)*kgrd(i,j)
        enddo
      enddo
c
c Calculate the gravity for unit density contrast for each grid:
      call gcfft(1,1) ! hgrd
      call gcfft(1,2) ! kgrd
      call gcfft(1,3) ! herr
      call gcfft(1,4) ! kerr
      call gcfft(1,5) ! cgrd
      call gcfft(1,6) ! tmgd
      call gcfft(1,7) ! kmgd
      do j=1,ngm
        if(j.le.ngm/2+1)then
          ad2=(td*dfloat(j-1))**2
        else
          ad2=(td*dfloat(ngm+1-j))**2
        endif
        do i=1,ngm
          if(i.le.ngm/2+1)then
            ad1=(td*dfloat(i-1))**2
          else
            ad1=(td*dfloat(ngm+1-i))**2
          endif
          k2d=dsqrt(ad1+ad2)
          k4=(ad1+ad2)**2
          fdnm=ro1-ro0+fr*k4/gacc
c Flexural integration of gravity; includes the Bouguer correction:
          fgrv=cgv*(cintg(k2d,drho)-ro0)
          if(k2d.ne.0d0)then
c Thermal flexure:
            fwt=-tmgd(i,j)/fdnm
c Bulk crustal lithologic flexure:
            fwk=-kmgd(i,j)/fdnm
c Moho-loading flexure:
            fwh=-hgrd(i,j)/fdnm
c           fwh=-(hgrd(i,j)-ffwt*fwt*falpha-ffwk*fwk*dkdrho)/fdnm
c           do ii=1,10
c             fwh=-(hgrd(i,j)-ffwh*fwh*drho)/fdnm
c             fwh=-(hgrd(i,j)-ffwt*fwt*falpha-ffwk*fwk*dkdrho-ffwh*fwh*
c    &            drho)/fdnm
c           enddo
c Gravity anomaly for unit Moho density contrast:
c           hgrd(i,j)=hgrd(i,j)*cgv*dexp(-k2d*mobar)+fgrv*fwh*ffwh
            hgrd(i,j)=hgrd(i,j)*cgv*dexp(-k2d*mobar)
c Flexural correction:
            hgrd(i,j)=hgrd(i,j)*(1d0-(1d0/(1d0+fr*k4/gacc)))
c Gravity error for unit Moho density contrast:
            herr(i,j)=herr(i,j)*cgv*dexp(-k2d*mobar)
c Gravity anomaly for unit Pratt density variation:
            kgrd(i,j)=kgrd(i,j)*cgv*(1d0-dexp(-k2d*mobar))/k2d
c Correction for thickness of bulk crust:
c    &          -cgrd(i,j)*cgv*dexp(-k2d*mobar)+fgrv*fwk*ffwk
     &          -cgrd(i,j)*cgv*dexp(-k2d*mobar)
c Flexural correction:
            kgrd(i,j)=kgrd(i,j)*(1d0-(1d0/(1d0+fr*k4/gacc)))
c Gravity error for unit Pratt density variation:
            kerr(i,j)=kerr(i,j)*cgv*(1d0-dexp(-k2d*mobar))/k2d
            tmgd(i,j)=fgrv*fwt*ffwt
            cgrd(i,j)=fwt+fwk+fwh
          else
            hgrd(i,j)=dcmplx(0d0,0d0)
            herr(i,j)=herr(i,j)*cgv
            kgrd(i,j)=dcmplx(0d0,0d0)
            kerr(i,j)=kerr(i,j)*cgv
            tmgd(i,j)=dcmplx(0d0,0d0)
            cgrd(i,j)=dcmplx(0d0,0d0)
          endif
        enddo
      enddo
      call gcfft(-1,1) ! hgrd
      call gcfft(-1,2) ! kgrd
      call gcfft(-1,3) ! herr
      call gcfft(-1,4) ! kerr
      call gcfft(-1,5) ! cgrd (flexure)
      call gcfft(-1,6) ! tmgd
c
      df=0d0
c     df=delg
      xwt=0d0
      do j=1,ngr
        do i=1,ngr
          if(tgrv(i,j).ne.-9999d0)then
c           tgrv(i,j)=tgrv(i,j)-ttbar+dreal(tmgd(i,j))
            tgrv(i,j)=tgrv(i,j)-ttbar
          else
            tgrv(i,j)=0d0
          endif
          sgrv(i,j)=sgrv(i,j)-ssbar
          gerr(i,j)=wtr(i,j)/((adr*dreal(herr(i,j)))**2+(dkdrho*
     &        dreal(kerr(i,j)))**2)
          xwt=xwt+gerr(i,j)
          df=df+gerr(i,j)*(gosg(i,j)-adr*dreal(hgrd(i,j))-dkdrho*
     &        dreal(kgrd(i,j))-tgrv(i,j))-sgrv(i,j)
        enddo
      enddo
      df=df/xwt
      do j=1,ngr
        do i=1,ngr
          ogrv(i,j)=df+adr*dreal(hgrd(i,j))+dkdrho*dreal(kgrd(i,j))+
     &        tgrv(i,j)+sgrv(i,j)
        enddo
      enddo
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c Gravity modeling: Residuals for H, K fields:
      grmn=1d20
      tck=tc(n)
      vvk=vv(n)
c
      do l=1,nkk
        vv(n)=kkmn+0.025d0*dfloat(l-1)
        do k=1,nhh
          tc(n)=hhmn+2.5d0*dfloat(k-1)
          mobar=0d0
          vvbar=0d0
c
c Interpolate crustal thickness and Vp/Vs to the gravity grid:
          do j=1,ngr
            do i=1,ngr
c Sort out 10 nearest points:
              call ignarst(lnr(i,j),ltr(i,j),4d4,10,0)
c Test for duplicate radii:
              ccs=dcos(ltr(i,j))*dcy(n)+dsin(ltr(i,j))*dsy(n)*
     &            dcos(lnr(i,j)-xl(n))
              ccs=dmax1(dmin1(ccs,1d0),-1d0)
              r=A*dacos(ccs)
              it=0
              do ii=1,10
                if(dabs(r-rnr(ii)).lt.1d0)it=ii
              enddo
              if(it.eq.0)then
c Set the tenth to point n:
                inr(10)=n
                rnr(10)=r
              else
c Replace duplicate radius in interpolation:
                inr(it)=n
                rnr(it)=r
              endif
c Interpolation, variance estimation at grid point:
              call krige10(10,hx,hr,kx,kr)
              hgrd(i,j)=dcmplx(hx,0d0)
              kgrd(i,j)=dcmplx(kx,0d0)
              herr(i,j)=dcmplx(hr,0d0)
              kerr(i,j)=dcmplx(kr,0d0)
              mobar=mobar+hx
              vvbar=vvbar+kx
c Mirrored grid assignment:
              hgrd(ngm+1-i,j)=dcmplx(hx,0d0)
              hgrd(i,ngm+1-j)=dcmplx(hx,0d0)
              hgrd(ngm+1-i,ngm+1-j)=dcmplx(hx,0d0)
              kgrd(ngm+1-i,j)=dcmplx(kx,0d0)
              kgrd(i,ngm+1-j)=dcmplx(kx,0d0)
              kgrd(ngm+1-i,ngm+1-j)=dcmplx(kx,0d0)
              herr(ngm+1-i,j)=dcmplx(hr,0d0)
              herr(i,ngm+1-j)=dcmplx(hr,0d0)
              herr(ngm+1-i,ngm+1-j)=dcmplx(hr,0d0)
              kerr(ngm+1-i,j)=dcmplx(kr,0d0)
              kerr(i,ngm+1-j)=dcmplx(kr,0d0)
              kerr(ngm+1-i,ngm+1-j)=dcmplx(kr,0d0)
            enddo
          enddo
          mobar=1d3*mobar/dfloat(ngr*ngr)
          vvbar=vvbar/dfloat(ngr*ngr)
c
          do j=1,ngm
            do i=1,ngm
c Compositional mass anomaly uses total crustal thickness; convert km to m:
              kmgd(i,j)=1d3*hgrd(i,j)
c Remove mean; convert km to m; z positive upwards:
              hgrd(i,j)=dcmplx(mobar,0d0)-1d3*hgrd(i,j)
              herr(i,j)=1d3*herr(i,j)
c Remove mean:
              kgrd(i,j)=kgrd(i,j)-dcmplx(vvbar,0d0)
c Correction of bulk crustal term for thickness variations:
              cgrd(i,j)=hgrd(i,j)*kgrd(i,j)
            enddo
          enddo
c
c Calculate the gravity for unit density contrast for each grid:
          call gcfft(1,1) ! hgrd
          call gcfft(1,2) ! kgrd
          call gcfft(1,3) ! herr
          call gcfft(1,4) ! kerr
          call gcfft(1,5) ! cgrd
          call gcfft(1,7) ! kmgd
          do j=1,ngm
            if(j.le.ngm/2+1)then
              ad2=(td*dfloat(j-1))**2
            else
              ad2=(td*dfloat(ngm+1-j))**2
            endif
            do i=1,ngm
              if(i.le.ngm/2+1)then
                ad1=(td*dfloat(i-1))**2
              else
                ad1=(td*dfloat(ngm+1-i))**2
              endif
              k2d=dsqrt(ad1+ad2)
              k4=(ad1+ad2)**2
              fdnm=ro1-ro0+fr*k4/gacc
c Flexural integration of gravity; includes the Bouguer correction:
              fgrv=cgv*(cintg(k2d,drho)-ro0)
              if(k2d.ne.0d0)then
c Bulk crustal lithologic flexure:
                fwk=-kmgd(i,j)/fdnm
c Moho-loading flexure:
                fwh=-hgrd(i,j)/fdnm
c               fwh=-(hgrd(i,j)-ffwt*fwt*falpha-ffwk*fwk*dkdrho)/fdnm
c               do ii=1,10
c                 fwh=-(hgrd(i,j)-ffwh*fwh*drho)/fdnm
c                 fwh=-(hgrd(i,j)-ffwt*fwt*falpha-ffwk*fwk*dkdrho-ffwh*
c    &                fwh*drho)/fdnm
c               enddo
c Gravity anomaly for unit Moho density contrast:
c               hgrd(i,j)=hgrd(i,j)*cgv*dexp(-k2d*mobar)+fgrv*fwh*ffwh
                hgrd(i,j)=hgrd(i,j)*cgv*dexp(-k2d*mobar)
c Flexural correction:
                hgrd(i,j)=hgrd(i,j)*(1d0-(1d0/(1d0+fr*k4/gacc)))
c Gravity error for unit Moho density contrast:
                herr(i,j)=herr(i,j)*cgv*dexp(-k2d*mobar)
c Gravity anomaly for unit Pratt density variation:
                kgrd(i,j)=kgrd(i,j)*cgv*(1d0-dexp(-k2d*mobar))/k2d
c Correction for thickness of bulk crust:
c    &              -cgrd(i,j)*cgv*dexp(-k2d*mobar)+fgrv*fwk*ffwk
     &              -cgrd(i,j)*cgv*dexp(-k2d*mobar)
c Flexural correction:
                kgrd(i,j)=kgrd(i,j)*(1d0-(1d0/(1d0+fr*k4/gacc)))
c Gravity error for unit Pratt density variation:
                kerr(i,j)=kerr(i,j)*cgv*(1d0-dexp(-k2d*mobar))/k2d
              else
                hgrd(i,j)=dcmplx(0d0,0d0)
                herr(i,j)=herr(i,j)*cgv
                kgrd(i,j)=dcmplx(0d0,0d0)
                kerr(i,j)=kerr(i,j)*cgv
              endif
            enddo
          enddo
          call gcfft(-1,1) ! hgrd
          call gcfft(-1,2) ! kgrd
          call gcfft(-1,3) ! herr
          call gcfft(-1,4) ! kerr
c
          df=0d0
c         df=delg
          xwt=0d0
          do j=1,ngr
            do i=1,ngr
              gerr(i,j)=wtr(i,j)/((adr*dreal(herr(i,j)))**2+
     &            (dkdrho*dreal(kerr(i,j)))**2)
              xwt=xwt+gerr(i,j)
              df=df+gerr(i,j)*(gosg(i,j)-adr*dreal(hgrd(i,j))-
     &            dkdrho*dreal(kgrd(i,j))-tgrv(i,j)-sgrv(i,j))
            enddo
          enddo
          df=df/xwt
          res=0d0
          do j=1,ngr
            do i=1,ngr
              gmd=df+dreal(adr*hgrd(i,j)+dkdrho*kgrd(i,j))+
     &            tgrv(i,j)+sgrv(i,j)
              rx=gosg(i,j)-gmd
              res=res+gerr(i,j)*rx*rx
              gmsm(i,j)=gmsm(i,j)+gmd
              gmss(i,j)=gmss(i,j)+gmd*gmd
            enddo
          enddo
          res=dsqrt(res/xwt)
          if(res.lt.grmn)then
            grmn=res
            hmn=tc(n)
            kmn=vv(n)
            do j=1,ngr
              do i=1,ngr
                hgrv(i,j)=df+adr*dreal(hgrd(i,j))
                kgrv(i,j)=df+dkdrho*dreal(kgrd(i,j))
                cgrv(i,j)=hgrv(i,j)+dkdrho*dreal(kgrd(i,j))
              enddo
            enddo
          endif
          write(12,*)tc(n)+el(n)/1d3,vv(n),res
          gres(k,l)=res
        enddo
      enddo
      close(12)
      tc(n)=tck
      vv(n)=vvk
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c Model variance estimate of N, number of observations
      mdn=dfloat(nhh*nkk)
      ssmx=0d0
      do j=1,ngr
        do i=1,ngr
          gmss(i,j)=wtr(i,j)*((mdn*gmss(i,j))-(gmsm(i,j)*gmsm(i,j)))/
     &        (mdn*(mdn-1d0))
          ssmx=dmax1(ssmx,gmss(i,j))
        enddo
      enddo
      parn=0d0
      do j=1,ngr
        do i=1,ngr
          parn=parn+gmss(i,j)/ssmx
        enddo
      enddo
c     print*,parn
      parn=4d-1*parn+6d-1*dfloat(ngr*ngr)
      parn=dmax1(parn,5d0)
c
      parm=3d0
c Contour file for confidence intervals on the gravity model:
      if(site(n)(6:10).eq.'     ')then
        open(unit=12,file='HK_Stacks/'//site(n)(1:5)//'_gcon.cnt',
     &      status='unknown')
      elseif(site(n)(7:10).eq.'    ')then
        open(unit=12,file='HK_Stacks/'//site(n)(1:6)//'_gcon.cnt',
     &      status='unknown')
      elseif(site(n)(8:10).eq.'   ')then
        open(unit=12,file='HK_Stacks/'//site(n)(1:7)//'_gcon.cnt',
     &      status='unknown')
      elseif(site(n)(9:10).eq.'  ')then
        open(unit=12,file='HK_Stacks/'//site(n)(1:8)//'_gcon.cnt',
     &      status='unknown')
      elseif(site(n)(10:10).eq.' ')then
        open(unit=12,file='HK_Stacks/'//site(n)(1:9)//'_gcon.cnt',
     &      status='unknown')
      else
        open(unit=12,file='HK_Stacks/'//site(n)//'_gcon.cnt',
     &      status='unknown')
      endif
      write(12,*)grmn,' C'
      x=grmn*dsqrt(1d0+parm*fcdfi(0.66d0,parn,parm)/(parn-parm))
      write(12,*)x,' C'
      x=grmn*dsqrt(1d0+parm*fcdfi(0.95d0,parn,parm)/(parn-parm))
      write(12,*)x,' C'
      close(12)
c Populate probability weighting array:
      prb(1)=grmn
      do i=2,100
        x=dfloat(i-1)/1d2
        prb(i)=grmn*dsqrt(1d0+parm*fcdfi(x,parn,parm)/(parn-parm))
      enddo
      do i=101,109
        x=(99d0+1d-1*dfloat(i-100))/1d2
        prb(i)=grmn*dsqrt(1d0+parm*fcdfi(x,parn,parm)/(parn-parm))
      enddo
      do i=110,118
        x=(99.9d0+1d-2*dfloat(i-109))/1d2
        prb(i)=grmn*dsqrt(1d0+parm*fcdfi(x,parn,parm)/(parn-parm))
      enddo
      do i=119,127
        x=(99.99d0+1d-3*dfloat(i-118))/1d2
        prb(i)=grmn*dsqrt(1d0+parm*fcdfi(x,parn,parm)/(parn-parm))
      enddo
c
c Save the best-fit gravity model to a file:
      if(site(n)(6:10).eq.'     ')then
        open(unit=12,file='HK_Stacks/'//site(n)(1:5)//'_grv.xyz',
     &      status='unknown')
      elseif(site(n)(7:10).eq.'    ')then
        open(unit=12,file='HK_Stacks/'//site(n)(1:6)//'_grv.xyz',
     &      status='unknown')
      elseif(site(n)(8:10).eq.'   ')then
        open(unit=12,file='HK_Stacks/'//site(n)(1:7)//'_grv.xyz',
     &      status='unknown')
      elseif(site(n)(9:10).eq.'  ')then
        open(unit=12,file='HK_Stacks/'//site(n)(1:8)//'_grv.xyz',
     &      status='unknown')
      elseif(site(n)(10:10).eq.' ')then
        open(unit=12,file='HK_Stacks/'//site(n)(1:9)//'_grv.xyz',
     &      status='unknown')
      else
        open(unit=12,file='HK_Stacks/'//site(n)//'_grv.xyz',
     &      status='unknown')
      endif
      rewind(12)
      do j=1,ngr
        do i=1,ngr
          write(12,'(f9.4,f8.4,6f7.1)')lnr(i,j)/dtr,9d1-ltr(i,j)/dtr,
     &        gosg(i,j),ogrv(i,j),hgrv(i,j),kgrv(i,j),cgrv(i,j),
     &        tgrv(i,j)
        enddo
      enddo
      close(12)
c
c Save the EARS site locations used for gravity modeling to a file:
      if(site(n)(6:10).eq.'     ')then
        open(unit=12,file='HK_Stacks/'//site(n)(1:5)//'_soi.ll',
     &      status='unknown')
      elseif(site(n)(7:10).eq.'    ')then
        open(unit=12,file='HK_Stacks/'//site(n)(1:6)//'_soi.ll',
     &      status='unknown')
      elseif(site(n)(8:10).eq.'   ')then
        open(unit=12,file='HK_Stacks/'//site(n)(1:7)//'_soi.ll',
     &      status='unknown')
      elseif(site(n)(9:10).eq.'  ')then
        open(unit=12,file='HK_Stacks/'//site(n)(1:8)//'_soi.ll',
     &      status='unknown')
      elseif(site(n)(10:10).eq.' ')then
        open(unit=12,file='HK_Stacks/'//site(n)(1:9)//'_soi.ll',
     &      status='unknown')
      else
        open(unit=12,file='HK_Stacks/'//site(n)//'_soi.ll',
     &      status='unknown')
      endif
      rewind(12)
      do i=1,nsts
        write(12,'(f9.4,f8.4)')xl(ists(i))/dtr,9d1-yl(ists(i))/dtr
      enddo
      close(12)
c
      end
