c***** krige10.for ***** version: 28 Dec 2010 **************************
c  Krige the values from nmax (ten or fewer) nearest sites.
c***********************************************************************
      subroutine krige10(nmax,hx,hr,kx,kr)
      implicit none
      include 'constants.inc'
c
c Arguments:
c
      integer nmax
      real*8 hr,hx,kr,kx
c
c Common Blocks:
c
      character*10 site(nsta)
      integer neq(nsta),nmeas
      real*8 el(nsta),cred(nsta)
      real*8 tc(nsta),tce(nsta),vv(nsta),vve(nsta),xl(nsta),yl(nsta)
      common/INDATA/site,el,cred,tc,tce,vv,vve,xl,yl,neq,nmeas
c
      integer inr(mint),nnt
      real*8 rnr(mint)
      common/NEARST/rnr,inr,nnt
c
      real*8 aradc,aradv,sillc,sillv,varc0,varv0
      common/SMVPAR/aradc,aradv,sillc,sillv,varc0,varv0
c
      real*8 dcy(nsta),dsy(nsta)
      common/STORCS/dcy,dsy
c
c Function Declarations:
c
      real*8 semv
c
c Local Variables:
c
      integer iwk(88)
      integer i,i1,i2,info,j,j1,j2,k,lwk,nnl,nnr,np
      real*8 ainv(11,11),amat(11,11),bvec(11),kwts(11)
      real*8 u(11,11),v(11,11),w(11),wk(1000)
      real*8 amn,amx,ccs,ks,r
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c Use nnr=nmax nearest points:
      if(nmax.gt.10)then
        write(6,*)'Subroutine krige10 interpolates from 10 pts or',
     &      ' less...'
        pause
      endif
      nnr=min(nnt,nmax)
      nnl=nnr+1
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c Initialize:
      do j=1,11
        do i=1,11
          amat(i,j)=0d0
          ainv(i,j)=0d0
          u(i,j)=0d0
          v(i,j)=0d0
        enddo
        bvec(j)=0d0
        kwts(j)=0d0
        w(j)=0d0
      enddo
      do i=1,88
        iwk(i)=0
      enddo
      do i=1,1000
        wk(i)=0d0
      enddo
c Kriging matrix/data vector for crustal thickness:
      do j=1,nnr
        do i=j,nnr
          ccs=dcy(inr(j))*dcy(inr(i))+dsy(inr(j))* dsy(inr(i))*
     &        dcos(xl(inr(j))-xl(inr(i)))
          ccs=dmax1(dmin1(ccs,1d0),-1d0)
          r=A*dacos(ccs)
          amat(i,j)=semv(r/aradc,varc0,sillc)
          amat(j,i)=amat(i,j)
        enddo
c Punctual kriging:
        amat(nnl,j)=1d0
        amat(j,nnl)=1d0
        bvec(j)=semv(rnr(j)/aradc,varc0,sillc)
      enddo
c Punctual kriging:
      bvec(nnl)=1d0
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c Solution for kriging weights requires a robust SVD algorithm:
      lwk=1000
      call dgesdd('A',nnl,nnl,amat,11,w,u,11,v,11,wk,lwk,iwk,info)
      if(info.ne.0)then
c       open(unit=47,file='Plots/tracks.dat',status='unknown',
        open(unit=47,file='Plots_DLS/tracks.dat',status='unknown',
     &      access='append')
        write(47,*)'Problem (krige10): info = ',info
        close(47)
      endif
c
c Invert w and zero singular values:
      np=0
      do i=1,nnl
        if(w(i).gt.tol)then
          w(i)=1d0/w(i)
        else
          w(i)=0d0
          np=i
        endif
      enddo
c
c Inverted matrix:
      do j=1,nnl
        do i=1,nnl
          do k=1,nnl
            ainv(i,j)=ainv(i,j)+v(k,i)*u(j,k)*w(k)
          enddo
        enddo
      enddo
c
      do j=1,nnl
        do i=1,nnl
          kwts(i)=kwts(i)+ainv(i,j)*bvec(j)
        enddo
      enddo
c
c Revise weights if singular valued:
      if(np.ne.0)then
        ks=0d0
        do i=1,nnr
          ks=ks+dabs(kwts(i))
        enddo
        do i=1,nnr
          kwts(i)=dabs(kwts(i))/ks
        enddo
      endif
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c Expected value & 1-sigma error of crustal thickness:
      hx=0d0
      hr=0d0
      amn=1d20
      amx=-1d20
      do i=1,nnr
        hx=hx+kwts(i)*tc(inr(i))
        hr=hr+kwts(i)*bvec(i)
        amn=dmin1(amn,tc(inr(i)))
        amx=dmax1(amx,tc(inr(i)))
      enddo
      if(hr+kwts(nnl).le.0d0)then
        hr=sillc
      else
        hr=dsqrt(hr+kwts(nnl))
      endif
      hx=dmax1(amn,dmin1(amx,hx))
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c Initialize:
      do j=1,11
        do i=1,11
          amat(i,j)=0d0
          ainv(i,j)=0d0
          u(i,j)=0d0
          v(i,j)=0d0
        enddo
        bvec(j)=0d0
        kwts(j)=0d0
        w(j)=0d0
      enddo
      do i=1,88
        iwk(i)=0
      enddo
      do i=1,1000
        wk(i)=0d0
      enddo
c Kriging matrix/data vector for Vp/Vs ratio:
      do j=1,nnr
        do i=j,nnr
          ccs=dcy(inr(j))*dcy(inr(i))+dsy(inr(j))*dsy(inr(i))*
     &        dcos(xl(inr(j))-xl(inr(i)))
          ccs=dmax1(dmin1(ccs,1d0),-1d0)
          r=A*dacos(ccs)
          amat(i,j)=semv(r/aradv,varv0,sillv)
          amat(j,i)=amat(i,j)
        enddo
c Punctual kriging:
        amat(nnl,j)=1d0
        amat(j,nnl)=1d0
        bvec(j)=semv(rnr(j)/aradv,varv0,sillv)
      enddo
c Punctual kriging:
      bvec(nnl)=1d0
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c Solution for kriging weights requires a robust SVD algorithm:
      lwk=1000
      call dgesdd('A',nnl,nnl,amat,11,w,u,11,v,11,wk,lwk,iwk,info)
      if(info.ne.0)then
c       open(unit=47,file='Plots/tracks.dat',status='unknown',
        open(unit=47,file='Plots_DLS/tracks.dat',status='unknown',
     &      access='append')
        write(47,*)'Problem (krige10): info = ',info
        close(47)
      endif
c
c Invert w and zero singular values:
      np=0
      do i=1,nnl
        if(w(i).gt.tol)then
          w(i)=1d0/w(i)
        else
          w(i)=0d0
          np=i
        endif
      enddo
c
c Inverted matrix:
      do j=1,nnl
        do i=1,nnl
          do k=1,nnl
            ainv(i,j)=ainv(i,j)+v(k,i)*u(j,k)*w(k)
          enddo
        enddo
      enddo
c
      do j=1,nnl
        do i=1,nnl
          kwts(i)=kwts(i)+ainv(i,j)*bvec(j)
        enddo
      enddo
c
c Revise weights if singular valued:
      if(np.ne.0)then
        ks=0d0
        do i=1,nnr
          ks=ks+dabs(kwts(i))
        enddo
        do i=1,nnr
          kwts(i)=dabs(kwts(i))/ks
        enddo
      endif
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c Expected value & 1-sigma error of Vp/Vs ratio:
      kx=0d0
      kr=0d0
      amn=1d20
      amx=-1d20
      do i=1,nnr
        kx=kx+kwts(i)*vv(inr(i))
        kr=kr+kwts(i)*bvec(i)
        amn=dmin1(amn,vv(inr(i)))
        amx=dmax1(amx,vv(inr(i)))
      enddo
      if(kr+kwts(nnl).le.0d0)then
        kr=sillv
      else
        kr=dsqrt(kr+kwts(nnl))
      endif
      kx=dmax1(amn,dmin1(amx,kx))
c
      return
      end

