c***** modatum.for ***** version: 28 Nov 2016 **************************
c Read in ETOPO5 integer elevation data and correct elevations at each 
c   site where EARS elevations are suspect.  Once elevations are clean,
c   correct "crustal thickness" estimates to sea level datum (more 
c   appropriate for comparison to gravity). Also, generate an elevation
c   file at the main gravity grid sampling to use for comparison with
c   flexural response terms in getdrho (i.e., to solve for a flexural
c   decoupling factor; filtered to remove wavelengths of zero flexure),
c   and keep etopo for use in gravmod.
c***********************************************************************
      subroutine modatum
      implicit none
      include 'constants.inc'
c
c Common Blocks:
c
      integer*2 etopo(4320,2160)
      real*8 elobs(ngs,ngs)
      common/ELVOBS/elobs,etopo
c
      integer gmaxs(ngs,ngs)
      real*8 gobs(ngs,ngs),gsed(ngs,ngs),gthm(ngs,ngs),lat(ngs,ngs)
      real*8 lon(ngs,ngs),mthm(ngs,ngs),wt(ngs,ngs),ww(ngs,ngs)
      common/GRVOBS/gobs,gsed,gthm,lat,lon,mthm,wt,ww,gmaxs
c
      character*10 site(nsta)
      integer neq(nsta),nmeas
      real*8 el(nsta),cred(nsta)
      real*8 tc(nsta),tce(nsta),vv(nsta),vve(nsta),xl(nsta),yl(nsta)
      common/INDATA/site,el,cred,tc,tce,vv,vve,xl,yl,neq,nmeas
c
      integer ntr
      complex*16 herr(ngmx,ngmx),kerr(ngmx,ngmx)
      complex*16 cgrd(ngmx,ngmx),hgrd(ngmx,ngmx),kgrd(ngmx,ngmx)
      complex*16 kmgd(ngmx,ngmx),tmgd(ngmx,ngmx),twgd(ngmx,ngmx)
      common/MODGRD/cgrd,herr,hgrd,kerr,kgrd,kmgd,tmgd,twgd,ntr
c
c Local Variables:
c
      integer ix(nsta),jy(nsta)
      integer i,ii,j,jj,k
      real*8 ad1,ad2,dd,fdnm,k2d,k4,td,te
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c Identify nearest ETOPO5 pixel location of each site:
      do k=1,nmeas
        if(xl(k).gt.0d0)then
          ix(k)=idnint(12d0*xl(k)/dtr)+1
        else
          ix(k)=idnint(12d0*(3.6d2+xl(k)/dtr))+1
        endif
        jy(k)=idnint(12d0*yl(k)/dtr)+1
      enddo
c Open the ETOPO5 elevation data file:
      open(unit=11,file='GravTopo/ETOPO5.asc',form='formatted',
     &    status='old')
      rewind(11)
c Read in the file and write if a match:
      do j=1,2160
        read(11,'(20i5)')(etopo(i,j),i=1,4320)
      enddo
      close(11)
      do k=1,nmeas
        if(site(k)(1:4).eq.'XF94')then
          el(k)=0.3048d0*el(k)
        elseif(el(k).gt.1.7d3.and.
     &      idnint(el(k)/dfloat(etopo(ix(k),jy(k)))).eq.3)then
          el(k)=0.3048d0*el(k)
        elseif(el(k).gt.5d3)then
          el(k)=0.3048d0*el(k)
        endif
        if(el(k).eq.0d0)then
          el(k)=dfloat(etopo(ix(k),jy(k)))
        endif
        if(el(k).eq.1.8d0)then
          el(k)=dfloat(etopo(ix(k),jy(k)))
        endif
        if(el(k).lt.-86d0)then
          el(k)=dfloat(etopo(ix(k),jy(k)))
        endif
        tc(k)=tc(k)-el(k)/1d3
      enddo
c Now interpolate elevation to grav grid:
      do j=1,ngs
        do i=1,ngs
          if(lon(i,j).ge.0d0)then
            ii=idnint(12d0*lon(i,j)/dtr)+1
          else
            ii=idnint(12d0*(3.6d2+lon(i,j)/dtr))+1
          endif
          jj=idnint(12d0*lat(i,j)/dtr)+1
          elobs(i,j)=dfloat(etopo(ii,jj))
        enddo
      enddo
c
c And filter to remove unequivocal surface load topo:
c
c     do j=1,ngs
c       do i=1,ngs
c         hgrd(i,j)=dcmplx(elobs(i,j),0d0)
c Mirrored grid assignment:
c         hgrd(ngmx+1-i,j)=dcmplx(elobs(i,j),0d0)
c         hgrd(i,ngmx+1-j)=dcmplx(elobs(i,j),0d0)
c         hgrd(ngmx+1-i,ngmx+1-j)=dcmplx(elobs(i,j),0d0)
c       enddo
c     enddo
c Calculate the flexurally filtered topography, Te = 5 km:
c     ntr=ngmx
c     td=2d0*pi/(dgm*dfloat(ngmx-1))
c     te=5d3
c     dd=8.89d9*te*te*te
c     call gcfft(1,1) ! hgrd
c     do j=1,ngmx
c       if(j.le.ngmx/2+1)then
c         ad2=(td*dfloat(j-1))**2
c       else
c         ad2=(td*dfloat(ngmx+1-j))**2
c       endif
c       do i=1,ngmx
c         if(i.le.ngmx/2+1)then
c           ad1=(td*dfloat(i-1))**2
c         else
c           ad1=(td*dfloat(ngmx+1-i))**2
c         endif
c         k2d=dsqrt(ad1+ad2)
c         k4=(ad1+ad2)**2
c         fdnm=1d0/(1d0+dd*k4/gacc)
c         if(k2d.ne.0d0)then
c           hgrd(i,j)=hgrd(i,j)*fdnm
c         else
c           hgrd(i,j)=dcmplx(0d0,0d0)
c         endif
c       enddo
c     enddo
c     call gcfft(-1,1) ! hgrd
c     open(unit=12,file='Plots_DLS/Etopo.lle',status='unknown')
c     rewind(12)
c     do j=1,ngs
c       do i=1,ngs
c         elobs(i,j)=dreal(hgrd(i,j))
c         write(12,'(f9.4,f8.4,f8.1)')lon(i,j)/dtr,9d1-lat(i,j)/
c    &        dtr,elobs(i,j)
c       enddo
c     enddo
c     close(12)
c
      return
      end
