c***** read_grav.for ***** version: 22 May 2018 ************************
c Read in the WGM2012 Bouguer gravity data from the region. Interpolate
c    to the western US map used for comparison in getdrho. Also read in 
c    estimates of thermal-derived gravity & mass, and effective elastic
c    thickness estimates for flexural modeling.
c***********************************************************************
      subroutine read_grav
      implicit none
      include 'constants.inc'
c
c Common Blocks:
c
      real*8 delg,drho,dkdrho,falpha,ffwh,ffwk,ffwt,frav,fsed
      common/DENPAR/delg,drho,dkdrho,falpha,ffwh,ffwk,ffwt,frav,fsed
c
      real*8 grav(ngi,ngj),rgdy(nti,ntj)
      common/GRAVIT/grav,rgdy
c
      integer gmaxs(ngs,ngs)
      real*8 gobs(ngs,ngs),gsed(ngs,ngs),gthm(ngs,ngs),lat(ngs,ngs)
      real*8 lon(ngs,ngs),mthm(ngs,ngs),wt(ngs,ngs),ww(ngs,ngs)
      common/GRVOBS/gobs,gsed,gthm,lat,lon,mthm,wt,ww,gmaxs
c
      integer ntr
      complex*16 herr(ngmx,ngmx),kerr(ngmx,ngmx)
      complex*16 cgrd(ngmx,ngmx),hgrd(ngmx,ngmx),kgrd(ngmx,ngmx)
      complex*16 kmgd(ngmx,ngmx),tmgd(ngmx,ngmx),twgd(ngmx,ngmx)
      common/MODGRD/cgrd,herr,hgrd,kerr,kgrd,kmgd,tmgd,twgd,ntr
c
      integer inr(mint),nnt
      real*8 rnr(mint)
      common/NEARST/rnr,inr,nnt
c
      real*8 dkdrho_std,drho_std,falpha_std,grms
      common/STOCHA/dkdrho_std,drho_std,falpha_std,grms
c
c Local Variables:
c
      character line*80
      integer*2 gra(ngi,ngj)
      integer i,i1,ii,j,j1,jj,mid,nip,nup
      real*8 rs(5),zs(5)
      real*8 ad1,ad2,ccs,cl,dcp,dsp,gbar,k2d,k4,lh,lm0,lo,lt,ph,ph1
      real*8 r1,r2,r3,r4,rho,td,te,th,wtn,x,y
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c Open the WGM2012 Bouguer gravity data file:
      open(unit=11,file='GravTopo/WGM2012.dat',status='old')
      rewind(11)
c Read in the data:
      wtn=0d0
      do i=1,5
        read(11,*)line
      enddo
1     read(11,*,end=2)lo,lt,ccs
        ii=idnint(30d0*(lo+144d0))+1
        jj=idnint(30d0*(64d0-lt))+1
        if(ii.ge.1.and.ii.le.ngi.and.jj.ge.1.and.jj.le.ngj)then
          grav(ii,jj)=ccs
          wtn=wtn+1d0
        endif
      goto1
2     close(11)
      write(6,*)wtn,ngi*ngj
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c Regrid and (Lambert azimuthal equal-area) project gravity to a
c    11-km spaced, 257x257 grid:
      ph1=38d0*dtr
      lm0=-95.5d0*dtr
c Centered at -95.5, 38:
      dcp=dcos(ph1)
      dsp=dsin(ph1)
      mid=(ngs+1)/2
      wtn=0d0
      gbar=0d0
      grms=0d0
      do j=1,ngs
        y=dgm*dfloat(j-mid)
        do i=1,ngs
          x=dgm*dfloat(i-mid)
          rho=dsqrt(x*x+y*y)
          cl=2d0*dasin(rho/(2d0*A))
          lo=lm0+datan2(x*dsin(cl),rho*dcp*dcos(cl)-y*dsp*dsin(cl))
          lt=dasin(dcos(cl)*dsp+y*dsin(cl)*dcp/rho)
          if(i.eq.mid.and.j.eq.mid)lt=ph1
          lh=pi/2d0-lt
          lon(i,j)=lo
          lat(i,j)=lh
          ii=idint(30d0*(lo/dtr+144d0))+1
          i1=min(ngi-1,max(1,ii))
          jj=idint(30d0*(64d0-lt/dtr))+1
          j1=min(ngj-1,max(1,jj))
          ph=dtr*(dfloat(i1-1)/30d0-144d0)
          th=dtr*(dfloat(j1-1)/30d0+26d0)
          ccs=dcos(lh)*dcos(th)+dsin(lh)*dsin(th)*dcos(lo-ph)
          ccs=dmax1(dmin1(ccs,1d0),-1d0)
          r1=A*dacos(ccs)
          ph=dtr*(dfloat(i1)/30d0-144d0)
          ccs=dcos(lh)*dcos(th)+dsin(lh)*dsin(th)*dcos(lo-ph)
          ccs=dmax1(dmin1(ccs,1d0),-1d0)
          r2=A*dacos(ccs)
          th=dtr*(dfloat(j1)/30d0+26d0)
          ccs=dcos(lh)*dcos(th)+dsin(lh)*dsin(th)*dcos(lo-ph)
          ccs=dmax1(dmin1(ccs,1d0),-1d0)
          r3=A*dacos(ccs)
          ph=dtr*(dfloat(i1-1)/30d0-144d0)
          ccs=dcos(lh)*dcos(th)+dsin(lh)*dsin(th)*dcos(lo-ph)
          ccs=dmax1(dmin1(ccs,1d0),-1d0)
          r4=A*dacos(ccs)
          gobs(i,j)=(r2*r3*r4*grav(i1,j1)+r1*r3*r4*grav(i1+1,j1)+
     &        r1*r2*r4*grav(i1+1,j1+1)+r1*r2*r3*grav(i1,j1+1))/
     &        (r2*r3*r4+r1*r3*r4+r1*r2*r4+r1*r2*r3)
          wt(i,j)=1d0
          wtn=wtn+1d0
          gbar=gbar+gobs(i,j)
          grms=grms+gobs(i,j)*gobs(i,j)
        enddo
      enddo
c
      grms=dsqrt((wtn*grms-gbar*gbar)/(wtn*(wtn-1d0)))
      gbar=gbar/wtn
      write(6,*)'Gravity statistics (regional map):'
      write(6,*)'Mean gravity anomaly: ',gbar,'; RMS: ',grms
c
c Plotfiles remove locations far from the nearest EARS site:
      do j=1,ngs
        do i=1,ngs
          gmaxs(i,j)=0
c Sort out 5 nearest points:
          call narst(lon(i,j),lat(i,j),5,0)
          if(rnr(1).gt.7d4.and.wt(i,j).eq.1d0)then
            wt(i,j)=0d0
          endif
          ww(i,j)=0d0
        enddo
      enddo
c
c Limit the inversion for density parameters to the region of 
c   conterminous seismic instrumentation:
c Seed assumes grid center (lon0,lat0) is within conterminous coverage:
      ww((ngs+1)/2,(ngs+1)/2)=1d0
      nup=1
      dowhile(nup.gt.0)
        nup=0
        do j=1,ngs
          do i=1,ngs
            nip=0
            if(wt(i,j).eq.1d0.and.ww(i,j).eq.0d0)then
              do jj=max0(1,j-1),min0(ngs,j+1)
                do ii=max0(1,i-1),min0(ngs,i+1)
                  if(ww(ii,jj).eq.1d0)then
                    ww(i,j)=1d0
                    nip=1
                  endif
                enddo
              enddo
            endif
            nup=nup+nip
          enddo
        enddo
      enddo
c
      wtn=0d0
      gbar=0d0
      grms=0d0
      do j=1,ngs
        do i=1,ngs
          if(ww(i,j).eq.1d0)then
            wtn=wtn+1d0
            gbar=gbar+gobs(i,j)
            grms=grms+gobs(i,j)*gobs(i,j)
          endif
        enddo
      enddo
c
      grms=dsqrt((wtn*grms-gbar*gbar)/(wtn*(wtn-1d0)))
      gbar=gbar/wtn
      write(6,*)'Gravity statistics (conterminous map):'
      write(6,*)'Mean gravity anomaly: ',gbar,'; RMS: ',grms
c
c Set density parameter variances to seed values:
      dkdrho_std=dkdrho_std0
      drho_std=drho_std0
      falpha_std=falpha_std0
c
c Read in files to remove thermal contribution to gravity:
      open(11,file='Thermal_Grav.llz',status='old')
c     open(11,file='Thermal_Grav_DLS.llz',status='old')
      rewind(11)
      open(12,file='Thermal_Mass.llz',status='old')
c     open(12,file='Thermal_Mass_DLS.llz',status='old')
      rewind(12)
      do j=1,ngs
        do i=1,ngs
          read(11,'(f9.4,f8.4,f7.1)')r1,r2,gthm(i,j)
          read(12,'(f9.4,f8.4,e15.7)')r1,r2,mthm(i,j)
        enddo
      enddo
      close(11)
      close(12)
c
c Read in file to remove sedimentary contribution to gravity:
      open(12,file='SedGravGrd.llz',status='old')
      rewind(12)
      do j=1,ngs
        do i=1,ngs
          read(12,'(f9.4,f8.4,f7.1)')lo,lt,gsed(i,j)
        enddo
      enddo
      close(12)
c
c Read in Marta's estimates of Te in North America:
      open(11,file='Te_NoAm.llz',status='old')
      rewind(11)
      frav=0d0
3     read(11,*,end=4)lo,lt,te
        ii=idnint(4d0*(lo+1.5d2))+1
        jj=idnint(4d0*(lt-11d0))+1
        te=dmin1(1d2,te)
        te=dmax1(5d0,te)
        if(lo.gt.-98d0.and.lo.lt.-82d0.and.lt.gt.34d0.and.lt.lt.51d0.
     &      and.te.lt.30d0)te=60d0
        frav=frav+te
        te=te*1d3
c Flexural rigidity for Young's modulus = 10^11 Pa, Poisson's ratio = 0.25
        rgdy(ii,jj)=8.89d9*te*te*te
      goto3
4     close(11)
      frav=1d3*frav/dfloat(nti*ntj)
c "Average" flexural rigidity:
      frav=8.89d9*frav**3
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c Flexurally filter gravity to remove isostatically compensated wavenumbers:
      ntr=ngmx
      td=2d0*pi/(dgm*dfloat(ngmx-1))
      do j=1,ngs
        do i=1,ngs
          hgrd(i,j)=dcmplx(gobs(i,j),0d0)
          kgrd(i,j)=dcmplx(gsed(i,j),0d0)
          cgrd(i,j)=dcmplx(gthm(i,j),0d0)
c Mirrored grid assignment:
          hgrd(ngmx+1-i,j)=dcmplx(gobs(i,j),0d0)
          kgrd(ngmx+1-i,j)=dcmplx(gsed(i,j),0d0)
          cgrd(ngmx+1-i,j)=dcmplx(gthm(i,j),0d0)
          hgrd(i,ngmx+1-j)=dcmplx(gobs(i,j),0d0)
          kgrd(i,ngmx+1-j)=dcmplx(gsed(i,j),0d0)
          cgrd(i,ngmx+1-j)=dcmplx(gthm(i,j),0d0)
          hgrd(ngmx+1-i,ngmx+1-j)=dcmplx(gobs(i,j),0d0)
          kgrd(ngmx+1-i,ngmx+1-j)=dcmplx(gsed(i,j),0d0)
          cgrd(ngmx+1-i,ngmx+1-j)=dcmplx(gthm(i,j),0d0)
        enddo
      enddo
      call gcfft(1,1) ! gobs
      call gcfft(1,2) ! gsed
      call gcfft(1,5) ! gthm
      do j=1,ngmx
        if(j.le.ngmx/2+1)then
          ad2=(td*dfloat(j-1))**2
        else
          ad2=(td*dfloat(ngmx+1-j))**2
        endif
        do i=1,ngmx
          if(i.le.ngmx/2+1)then
            ad1=(td*dfloat(i-1))**2
          else
            ad1=(td*dfloat(ngmx+1-i))**2
          endif
          k2d=dsqrt(ad1+ad2)
          k4=(ad1+ad2)**2
          hgrd(i,j)=hgrd(i,j)*(1d0-(1d0/(1d0+frav*k4/gacc)))
          kgrd(i,j)=kgrd(i,j)*(1d0-(1d0/(1d0+frav*k4/gacc)))
          cgrd(i,j)=cgrd(i,j)*(1d0-(1d0/(1d0+frav*k4/gacc)))
        enddo
      enddo
      call gcfft(-1,1) ! gobs
      call gcfft(-1,2) ! gsed
      call gcfft(-1,5) ! gthm
      do j=1,ngs
        do i=1,ngs
          gobs(i,j)=dreal(hgrd(i,j))
          gsed(i,j)=dreal(kgrd(i,j))
          gthm(i,j)=dreal(cgrd(i,j))
        enddo
      enddo
c
      return
      end

